@@ -214,7 +214,10 @@ def func_name(
     try:
         if human and inspect.isbuiltin(fn):
             return str(fn)
-        fn_name = fn.__qualname__ if fqdn else fn.__name__
+        # Not possible for all objects to fake ``__qualname__``.
+        fn_name = (
+            fn.__qualname__ if fqdn and hasattr(fn, "__qualname__") else fn.__name__
+        )
         assert fn_name
 
         mod_name = getattr(fn, "__module__", None)
@@ -293,108 +296,6 @@ def func_sourcelines(fn, default=..., human=None) -> Optional[Tuple[str, int]]:
         return default
 
 
-def jetsam(ex, locs, *salvage_vars: str, annotation="jetsam", **salvage_mappings):
-    """
-    Annotate exception with salvaged values from locals() and raise!
-
-    :param ex:
-        the exception to annotate
-    :param locs:
-        ``locals()`` from the context-manager's block containing vars
-        to be salvaged in case of exception
-
-        ATTENTION: wrapped function must finally call ``locals()``, because
-        *locals* dictionary only reflects local-var changes after call.
-    :param annotation:
-        the name of the attribute to attach on the exception
-    :param salvage_vars:
-        local variable names to save as is in the salvaged annotations dictionary.
-    :param salvage_mappings:
-        a mapping of destination-annotation-keys --> source-locals-keys;
-        if a `source` is callable, the value to salvage is retrieved
-        by calling ``value(locs)``.
-        They take precedence over`salvage_vars`.
-
-    :raises:
-        any exception raised by the wrapped function, annotated with values
-        assigned as attributes on this context-manager
-
-    - Any attributes attached on this manager are attached as a new dict on
-      the raised exception as new  ``jetsam`` attribute with a dict as value.
-    - If the exception is already annotated, any new items are inserted,
-      but existing ones are preserved.
-
-    **Example:**
-
-    Call it with managed-block's ``locals()`` and tell which of them to salvage
-    in case of errors::
-
-
-        try:
-            a = 1
-            b = 2
-            raise Exception()
-        exception Exception as ex:
-            jetsam(ex, locals(), "a", b="salvaged_b", c_var="c")
-            raise
-
-    And then from a REPL::
-
-        import sys
-        sys.last_value.jetsam
-        {'a': 1, 'salvaged_b': 2, "c_var": None}
-
-    ** Reason:**
-
-    Graphs may become arbitrary deep.  Debugging such graphs is notoriously hard.
-
-    The purpose is not to require a debugger-session to inspect the root-causes
-    (without precluding one).
-
-    Naively salvaging values with a simple try/except block around each function,
-    blocks the debugger from landing on the real cause of the error - it would
-    land on that block;  and that could be many nested levels above it.
-    """
-    ## Fail EARLY before yielding on bad use.
-    #
-    assert isinstance(ex, Exception), ("Bad `ex`, not an exception dict:", ex)
-    assert isinstance(locs, dict), ("Bad `locs`, not a dict:", locs)
-    assert all(isinstance(i, str) for i in salvage_vars), (
-        "Bad `salvage_vars`!",
-        salvage_vars,
-    )
-    assert salvage_vars or salvage_mappings, "No `salvage_mappings` given!"
-    assert all(isinstance(v, str) or callable(v) for v in salvage_mappings.values()), (
-        "Bad `salvage_mappings`:",
-        salvage_mappings,
-    )
-
-    ## Merge vars-mapping to save.
-    for var in salvage_vars:
-        if var not in salvage_mappings:
-            salvage_mappings[var] = var
-
-    try:
-        annotations = getattr(ex, annotation, None)
-        if not isinstance(annotations, dict):
-            annotations = {}
-            setattr(ex, annotation, annotations)
-
-        ## Salvage those asked
-        for dst_key, src in salvage_mappings.items():
-            try:
-                salvaged_value = src(locs) if callable(src) else locs.get(src)
-                annotations.setdefault(dst_key, salvaged_value)
-            except Exception as ex:
-                log.warning(
-                    "Suppressed error while salvaging jetsam item (%r, %r): %r"
-                    % (dst_key, src, ex)
-                )
-    except Exception as ex2:
-        log.warning("Suppressed error while annotating exception: %r", ex2, exc_info=1)
-        raise ex2
-
-
 class PlotArgs(NamedTuple):
     """
     All the args of a :meth:`.Plottable.plot()` call,
@@ -705,7 +606,7 @@ class Plottable(abc.ABC):
             digraph pipeline {
             fontname=italic;
             label=<pipeline>;
-            <a> [fillcolor=wheat, label=a, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+            <a> [fillcolor=wheat, fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
             ...
 
         .. graphtik::
@@ -770,6 +671,9 @@ class Plottable(abc.ABC):
             else None
         )
 
+        if plot_args.steps is None and plot_args.solution is not None:
+            plot_args = plot_args._replace(steps=plot_args.solution.plan.steps)
+
         return plot_args.plotter.plot(plot_args)
 
     @abc.abstractmethod
@@ -813,9 +717,11 @@ class Plottable(abc.ABC):
             if isinstance(n, Operation) and predicate(n, d)
         ]
 
-    def find_op_by_name(self, name) -> Union["Operation", None]:
+    def find_op_by_name(self, name) -> Optional["Operation"]:
         """Fetch the 1st operation named with the given `name`."""
-        for n in self.ops:
+        from .planning import yield_ops
+
+        for n in yield_ops(self.graph):
             if n.name == name:
                 return n
 
@@ -856,10 +762,6 @@ class Operation(Plottable, abc.ABC):
     provides: Items
     op_provides: Items
 
-    @property
-    def __name__(self) -> str:
-        return self.name
-
     @abc.abstractmethod
     def compute(self, named_inputs, outputs=None):
         """
