@@ -73,23 +73,20 @@ Like :func:`set_debug()` as a context-manager, resetting back to old value.
 .. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_debug = partial(_getter, _debug)
-"""see :func:`set_debug()`"""
-set_debug = partial(_tristate_set, _debug)
 """
-When true, increase details on string-representation of network objects and errors.
+Return :func:`.set_debug` or `True` if :envvar:`GRAPHTIK_DEBUG` not one of ``0 false off no``.
 
-:param enabled:
-    - ``None, False, string(0, false, off, no)``: Disabled
-    - TODO: ``1``: Enable ALL ``DEBUG_XXX``
-    - TODO: integers: Enable respective ``DEBUG_XXX`` bit-field constants
-    - anything else: Enable ALL ``DEBUG_XXX``
-
-Affected behavior:
+Affected behavior when :ref:`debug` enabled:
 
 .. debug-behavior-start
 
++ on errors, plots the 1st errored solution/plan/pipeline/net (in that order)
+  in an SVG file inside the temp-directory, and its path is logged in ERROR-level;
++ :term:`jetsam` logs in ERROR (instead of in DEBUG) all annotations on all calls
+  up the stack trace (logged from ``graphtik.jetsam.err`` logger);
 + :meth:`FnOp.compute()` prints out full given-inputs (not just their keys);
-+ net objects print details recursively;
++ net objects print more details recursively, like fields (not just op-names) and
+  prune-comments;
 + plotted SVG diagrams include style-provenance as tooltips;
 + Sphinx extension also saves the original DOT file next to each image
   (see :confval:`graphtik_save_dot_files`).
@@ -106,6 +103,16 @@ exceptions, doctests), not just debug statements (also affected by this flag).
 :return:
     a "reset" token (see :meth:`.ContextVar.set`)
 """
+set_debug = partial(_tristate_set, _debug)
+"""
+Enable/disable debug-mode.
+
+:param enabled:
+    - ``None, False, string(0, false, off, no)``: Disabled
+    - anything else: Enable DEBUG
+
+see :func:`is_debug()`
+"""
 
 
 def abort_run():
