@@ -3,6 +3,7 @@
 """:term:`execute` the :term:`plan` to derrive the :term:`solution`."""
 import logging
 import random
+import sys
 import time
 from collections import ChainMap, abc, defaultdict, namedtuple
 from contextvars import ContextVar, copy_context
@@ -24,7 +25,6 @@ from .base import (
     aslist,
     astuple,
     first_solid,
-    jetsam,
 )
 from .config import (
     get_execution_pool,
@@ -37,6 +37,7 @@ from .config import (
     is_reschedule_operations,
     is_skip_evictions,
 )
+from .jetsam import save_jetsam
 from .modifier import (
     acc_contains,
     acc_delitem,
@@ -49,6 +50,7 @@ from .modifier import (
     is_sfx,
 )
 from .planning import (
+    OpMap,
     unsatisfied_operations,
     yield_chaindocs,
     yield_node_names,
@@ -100,9 +102,11 @@ class Solution(ChainMap, Plottable):
         #: - value None:        execution ok
         #: - value Exception:   execution failed
         #: - value Collection:  canceled provides
-        self.executed = {}
-        #: A sorted set of :term:`canceled operation`\\s due to upstream failures.
-        self.canceled = iset()  # not iterated, order not important, but ...
+        self.executed: OpMap = {}
+        # not iterated, order unimportant, but ...
+        #: A {op, prune-explanation} dictionary with :term:`canceled operation`\\s
+        #: due to upstream failures.
+        self.canceled: OpMap = {}
         self.elapsed_ms = {}
         #: A unique identifier to distinguish separate flows in execution logs.
         self.solid = "%X" % random.randint(0, 2 ** 16)
@@ -155,7 +159,7 @@ class Solution(ChainMap, Plottable):
         :param op:
             for logging
         """
-        canceled = unsatisfied_operations(
+        canceled, _sorted_nodes = unsatisfied_operations(
             dag,
             [
                 i
@@ -165,8 +169,8 @@ class Solution(ChainMap, Plottable):
             ],
         )
         # Minus executed, bc partial-out op might not have any provides left.
-        newly_canceled = iset(canceled) - self.canceled - self.executed
-        self.canceled.update(newly_canceled)
+        newly_canceled = canceled.keys() - self.canceled.keys() - self.executed.keys()
+        self.canceled.update((k, canceled[k]) for k in newly_canceled)
 
         if log.isEnabledFor(logging.INFO):
             log.info(
@@ -193,7 +197,7 @@ class Solution(ChainMap, Plottable):
             for ac, kv in zip(accessors, outputs.items()):
                 update_groups[ac and ac.update].append(kv)
 
-            ## Values without :attr:`Accessor.update` are to be set one-by-one,
+            ## Values without :attr:`._accessor.update` are to be set one-by-one,
             #  further below.
             outputs = update_groups.pop(None, None)
 
@@ -295,7 +299,7 @@ class Solution(ChainMap, Plottable):
     @property
     def overwrites(self) -> Mapping[Any, List]:
         """
-        The data in the solution that exist more than once.
+        The data in the solution that exist more than once (refreshed on every call).
 
         A "virtual" property to a dictionary with keys the names of values that
         exist more than once, and values, all those values in a list, ordered
@@ -316,7 +320,7 @@ class Solution(ChainMap, Plottable):
         }
         incomplete = iset(chain(self.canceled, failures.keys()))
         if incomplete:
-            incomplete = [op.name for op in incomplete]
+            incomplete = list(yield_node_names(incomplete))
             partial_msgs = {
                 f"\n  +--{op.name}: {list(pouts)}"
                 for op, pouts in self.executed.items()
@@ -427,7 +431,8 @@ def _do_task(task):
 
 
 class ExecutionPlan(
-    namedtuple("ExecPlan", "net needs provides dag steps asked_outs"), Plottable
+    namedtuple("ExecPlan", "net needs provides dag steps asked_outs comments"),
+    Plottable,
 ):
     """
     A pre-compiled list of operation steps that can :term:`execute` for the given inputs/outputs.
@@ -457,6 +462,9 @@ class ExecutionPlan(
 
         When true, :term:`eviction`\\s may kick in (unless disabled by :term:`configurations`),
         otherwise, *evictions* (along with prefect-evictions check) are skipped.
+    .. attribute:: comments
+
+        an {op, prune-explanation} dictionary
     """
 
     @property
@@ -485,33 +493,54 @@ class ExecutionPlan(
     def __repr__(self):
         needs = aslist(self.needs, "needs")
         provides = aslist(self.provides, "provides")
-        steps = (
-            "".join(f"\n  +--{s}" for s in self.steps)
+        steps = self.steps or ()
+        steps_str = (
+            "".join(f"\n  +--{s}" for s in steps)
             if is_debug()
-            else ", ".join(yield_node_names(self.steps))
+            else ", ".join(yield_node_names(steps))
+        )
+        comments = (
+            (f"\n  +--prune-comments: {self.comments}" if is_debug() else "")
+            if self.comments
+            else ""
         )
-        return f"ExecutionPlan(needs={needs}, provides={provides}, x{len(self.steps)} steps: {steps})"
 
-    def validate(self, inputs: Items, outputs: Items):
+        return (
+            f"ExecutionPlan(needs={needs}, provides={provides}"
+            f", x{len(steps)} steps: {steps_str}{comments})"
+        )
+
+    def validate(self, inputs: Items = UNSET, outputs: Items = UNSET):
         """
         Scream on invalid inputs, outputs or no operations in graph.
 
+        :param inputs:
+            the inputs that this plan was :term:`compile`\\d for, or MORE;
+            will scream if LESS...
+        :param outputs:
+            the outputs that this plan was :term:`compile`\\d for, or LESS;
+            will scream if MORE...
+
         :raises ValueError:
-            - If cannot produce any `outputs` from the given `inputs`, with msg:
-
-                *Unsolvable graph: ...*
-
-            - If given `inputs` mismatched plan's :attr:`needs`, with msg:
-
-                *Plan needs more inputs...*
-
-            - If net cannot produce asked `outputs`, with msg:
-
-                *Unreachable outputs...*
+            *Unsolvable graph...*
+                if it cannot produce any `outputs` from the given `inputs`.
+            *Plan needs more inputs...*
+                if given `inputs` mismatched plan's :attr:`needs`.
+            *Unreachable outputs...*
+                if net cannot produce asked `outputs`.
 
         """
         if not self.dag:
-            raise ValueError(f"Unsolvable graph:\n  {self}")
+            raise ValueError(
+                f"Unsolvable graph:\n  +--{self.net}"
+                f"\n  +--possible inputs: {list(self.net.needs)}"
+                "\n  +--possible outputs: {list(self.net.provides)}"
+            )
+
+        if inputs is UNSET:
+            inputs = self.needs
+        if outputs is UNSET:
+            outputs = self.provides
 
         # Check plan<-->inputs mismatch.
         #
@@ -553,6 +582,7 @@ class ExecutionPlan(
         input_values = dict(solution)
 
         def prep_task(op):
+            ok = False
             try:
                 # Mark start time here, to include also marshalling overhead.
                 solution.elapsed_ms[op] = time.time()
@@ -573,10 +603,12 @@ class ExecutionPlan(
                     task = partial(_do_task, task)
                     task.get = task.__call__
 
+                ok = True
                 return task
-            except Exception as ex:
-                jetsam(ex, locals(), "task", plan="self")
-                raise
+            finally:
+                if not ok:
+                    ex = sys.exc_info()[1]
+                    save_jetsam(ex, locals(), "task", plan="self")
 
         return [prep_task(op) for op in operations]
 
@@ -605,7 +637,7 @@ class ExecutionPlan(
             solution.operation_executed(op, outputs)
 
             elapsed = elapsed_ms(op)
-            log.debug(
+            log.info(
                 "... (%s) op(%s) completed in %sms.", solution.solid, op.name, elapsed
             )
         except Exception as ex:
@@ -632,7 +664,7 @@ class ExecutionPlan(
             else:
                 # Although `plan` have added to jetsam in `compute()``,
                 # add it again, in case compile()/execute() is called separately.
-                jetsam(ex, locals(), "solution", task="future", plan="self")
+                save_jetsam(ex, locals(), "solution", task="future", plan="self")
                 raise
 
     def _execute_thread_pool_barrier_method(self, solution: Solution):
@@ -688,8 +720,8 @@ class ExecutionPlan(
                             )
                         )
                     ):
-                        if _isDebugLogging():
-                            log.debug(
+                        if log.isEnabledFor(logging.INFO):
+                            log.info(
                                 "... (%s) evicting '%s' from solution%s.",
                                 solution.solid,
                                 node,
@@ -743,7 +775,7 @@ class ExecutionPlan(
             elif isinstance(step, str):
                 # Cache value may be missing if it is optional.
                 if step in solution:
-                    log.debug(
+                    log.info(
                         "... (%s) evicting '%s' from solution%s.",
                         solution.solid,
                         step,
@@ -792,18 +824,14 @@ class ExecutionPlan(
             +1 for inputs in separate dictionaries.
 
         :raises ValueError:
-            - If plan does not contain any operations, with msg:
-
-                *Unsolvable graph: ...*
-
-            - If given `inputs` mismatched plan's :attr:`needs`, with msg:
-
-                *Plan needs more inputs...*
-
-            - If net cannot produce asked `outputs`, with msg:
-
-                *Unreachable outputs...*
+            *Unsolvable graph...*
+                if it cannot produce any `outputs` from the given `inputs`.
+            *Plan needs more inputs...*
+                if given `inputs` mismatched plan's :attr:`needs`.
+            *Unreachable outputs...*
+                if net cannot produce asked `outputs`.
         """
+        ok = False
         try:
             self.validate(named_inputs, outputs)
             dag = self.dag  # locals opt
@@ -836,7 +864,7 @@ class ExecutionPlan(
                 layered_solution=layered_solution,
             )
 
-            log.debug(
+            log.info(
                 "=== (%s) Executing pipeline(%s)%s%s, on inputs%s, according to %s...",
                 solution.solid,
                 name,
@@ -846,19 +874,19 @@ class ExecutionPlan(
                 self,
             )
 
-            ok = False
+            ok2 = False
             try:
                 executor(solution)
-                ok = True
+                ok2 = True
             finally:
                 ## Log cumulative operations elapsed time.
                 #
-                if _isDebugLogging():
+                if log.isEnabledFor(logging.INFO):
                     elapsed = sum(solution.elapsed_ms.values())
-                    log.debug(
+                    log.info(
                         "=== (%s) %s pipeline(%s) in %0.3fms.",
                         solution.solid,
-                        "Completed" if ok else "FAILED",
+                        "Completed" if ok2 else "FAILED",
                         name,
                         elapsed,
                     )
@@ -878,7 +906,9 @@ class ExecutionPlan(
                     "\n  (tip: enable DEBUG-logging and/or set GRAPHTIK_DEBUG envvar to investigate)"
                 )
 
+            ok = True
             return solution
-        except Exception as ex:
-            jetsam(ex, locals(), "solution")
-            raise
+        finally:
+            if not ok:
+                ex = sys.exc_info()[1]
+                save_jetsam(ex, locals(), "solution")
