@@ -12,13 +12,13 @@
 import itertools as itt
 import logging
 import textwrap
+import sys
 from collections import abc as cabc
-from functools import wraps
+from functools import update_wrapper, wraps
 from typing import (
     Any,
     Callable,
     Collection,
-    Hashable,
     List,
     Mapping,
     Tuple,
@@ -30,28 +30,29 @@ from .base import (
     UNSET,
     Items,
     Operation,
-    Plottable,
     PlotArgs,
+    Plottable,
     Token,
     aslist,
     astuple,
     debug_var_tip,
     first_solid,
     func_name,
-    jetsam,
 )
+from .jetsam import save_jetsam
 from .modifier import (
     dep_renamed,
     dep_singularized,
     dep_stripped,
     get_keyword,
+    is_implicit,
     is_optional,
     is_pure_sfx,
     is_sfx,
     is_sfxed,
     is_vararg,
     is_varargs,
-    jsonp,
+    modify,
     optional,
 )
 
@@ -120,7 +121,7 @@ def as_renames(i, argname):
 
 
 def jsonp_ize(dep):
-    return jsonp(dep) if "/" in dep and type(dep) is str else dep
+    return modify(dep) if "/" in dep and type(dep) is str else dep
 
 
 def jsonp_ize_all(deps):
@@ -133,7 +134,7 @@ def jsonp_ize_all(deps):
 def reparse_operation_data(
     name, needs, provides, aliases=()
 ) -> Tuple[
-    Hashable, Collection[str], Collection[str], Collection[Tuple[str, str]],
+    str, Collection[str], Collection[str], Collection[Tuple[str, str]],
 ]:
     """
     Validate & reparse operation data as lists.
@@ -144,8 +145,6 @@ def reparse_operation_data(
     As a separate function to be reused by client building operations,
     to detect errors early.
     """
-    # if name is not None and not name or not isinstance(name, cabc.Hashable):
-    #     raise TypeError(f"Operation `name` must be a truthy hashable, got: {name}")
     if name is not None and not isinstance(name, str):
         raise TypeError(f"Non-str `name` given: {name}")
 
@@ -176,13 +175,14 @@ def reparse_operation_data(
 
         aliases = [(jsonp_ize(src), jsonp_ize(dst)) for src, dst in aliases]
         alias_src = iset(src for src, _dst in aliases)
-        if not alias_src <= set(provides):
+        stripped_provides = iset(dep_stripped(d) for d in provides)
+        if not alias_src <= stripped_provides:
             bad_alias_sources = alias_src - provides
             bad_aliases = ", ".join(
                 f"{src!r}-->{dst!r}" for src, dst in aliases if src in bad_alias_sources
             )
             raise ValueError(
-                f"The `aliases` [{bad_aliases}] rename non-existent provides in {list(provides)}!"
+                f"The `aliases` [{bad_aliases}] rename non-existent provides in {list(stripped_provides)}!"
             )
         sfx_aliases = [
             f"{src} -> {dst}" for src, dst in aliases if is_sfx(src) or is_sfx(dst)
@@ -200,32 +200,36 @@ def _spread_sideffects(
     deps: Collection[str],
 ) -> Tuple[Collection[str], Collection[str]]:
     """
-    Build fn/op dependencies from user ones by stripping or singularizing any :term:`sideffects`.
+    Build fn/op dependencies by stripping/singularizing any :term:`implicit`/:term:`sideffects`.
 
     :return:
-        the given `deps` duplicated as ``(fn_deps,  op_deps)``, where any instances of
+        the given `deps` duplicated as ``(op_deps, fn_deps)``, where any instances of
         :term:`sideffects` are processed like this:
 
-        `fn_deps`
-            - any :func:`.sfxed` are replaced by the :func:`stripped <.dep_stripped>`
-              dependency consumed/produced by underlying functions, in the order
-              they are first met (the rest duplicate `sideffected` are discarded).
-            - any :func:`.sfx` are simply dropped;
-
         `op_deps`
-            any :func:`.sfxed` are replaced by a sequence of ":func:`singularized
-            <.dep_singularized>`" instances, one for each item in their
-            :attr:`._Modifier.sfx_list` attribute, in the order they are first met
-            (any duplicates are discarded, order is irrelevant, since they don't reach
-            the function);
+            - any :func:`.sfxed` is replaced by a sequence of ":func:`singularized
+              <.dep_singularized>`" instances, one for each item in its
+              :term:`sfx_list`;
+            - any duplicates are discarded;
+            - order is irrelevant, since they don't reach the function.
+
+        `fn_deps`
+            - the dependencies consumed/produced by underlying functions, in the order
+              they are first met.  In particular, it replaces any :func:`.sfxed`
+              by the :func:`stripped <.dep_stripped>`, unless ...
+            - it had been declared as :term:`implicit`, in which case, it is discared;
+            - discards all the rest duplicate `sideffected` dependencies;
+            - any :func:`.sfx` are simply dropped.
     """
 
     #: The dedupe  any `sideffected`.
     seen_sideffecteds = set()
 
-    def strip_sideffecteds(dep):
-        """Strip and dedupe any sfxed, drop any sfx. """
-        if is_sfxed(dep):
+    def strip_unique(dep):
+        """Strip and dedupe any sfxed, drop any sfx and implicit. """
+        if is_implicit(dep):
+            pass
+        elif is_sfxed(dep):
             dep = dep_stripped(dep)
             if not dep in seen_sideffecteds:
                 seen_sideffecteds.add(dep)
@@ -238,7 +242,7 @@ def _spread_sideffects(
 
     if deps:
         deps = tuple(nn for n in deps for nn in dep_singularized(n))
-        fn_deps = tuple(nn for n in deps for nn in strip_sideffecteds(n))
+        fn_deps = tuple(nn for n in deps for nn in strip_unique(n))
         return deps, fn_deps
     else:
         return deps, deps
@@ -341,6 +345,22 @@ class FnOp(Operation):
         #: :seealso: :ref:`operation-nesting`
         self.name = name
 
+        #: Fake function attributes.
+        #:
+        if fn:
+            update_wrapper(
+                self,
+                fn,
+                assigned=("__module__", "__doc__", "__annotations__"),
+                updated=(),
+            )
+        self.__name__ = name
+        qname = getattr(fn, "__qualname__", None) or name
+        if qname:
+            # "ab.cd" => "ab.NAME", "ab" => "NAME", "" => "NAME"
+            qname = ".".join((*qname.split(".")[:-1], name))
+        self.__qualname__ = qname
+
         #: The :term:`needs` almost as given by the user
         #: (which may contain MULTI-sideffecteds and dupes),
         #: roughly morphed into `_fn_provides` + sideffects
@@ -541,6 +561,7 @@ class FnOp(Operation):
         But usually a callable is more practical, like the one below renaming
         only data names:
 
+            >>> from graphtik.modifier import dep_renamed
             >>> op.withset(renamer=lambda ren_args:
             ...            dep_renamed(ren_args.name, lambda n: f"parent.{n}")
             ...            if ren_args.typ != 'op' else
@@ -597,23 +618,21 @@ class FnOp(Operation):
             ner += 1
         if varargs_bad:
             errors.append(
-                f"{ner}. Expected needs{list(varargs_bad)} to be non-str iterables!"
+                f"{ner}. Expected varargs inputs to be non-str iterables:"
+                f" { {k: named_inputs[k] for k in varargs_bad} }"
             )
         inputs = dict(named_inputs) if is_debug() else list(named_inputs)
         errors.append(f"+++inputs: {inputs}")
-        errors.append(f"+++{self}")
 
-        msg = textwrap.indent("\n".join(errors), " " * 4)
-        raise ValueError(f"Failed preparing needs: \n{msg}")
+        return textwrap.indent("\n".join(errors), " " * 4)
 
     def _match_inputs_with_fn_needs(self, named_inputs) -> Tuple[list, list, dict]:
-        positional, vararg_vals = [], []
-        kwargs = {}
+        positional, vararg_vals, kwargs = [], [], {}
         missing, varargs_bad = [], []
         for n in self._fn_needs:
             try:
                 ok = False
-                assert not is_sfx(n), locals()
+                assert not is_sfx(n) and not is_implicit(n), locals()
                 if n not in named_inputs:
                     if not is_optional(n) or is_sfx(n):
                         # It means `inputs` < compulsory `needs`.
@@ -621,6 +640,7 @@ class FnOp(Operation):
                         # but ..?
                         ##
                         missing.append(n)
+                    ok = True
                     continue
                 else:
                     inp_value = named_inputs[n]
@@ -650,68 +670,132 @@ class FnOp(Operation):
                     )
 
         if missing or varargs_bad:
-            raise self._prepare_match_inputs_error(missing, varargs_bad, named_inputs)
+            msg = self._prepare_match_inputs_error(missing, varargs_bad, named_inputs)
+            raise ValueError(f"Failed matching inputs <=> needs for {self}: \n{msg}")
 
         return positional, vararg_vals, kwargs
 
+    def _zip_results_returns_dict(self, results, is_rescheduled) -> dict:
+        if hasattr(results, "_asdict"):  # named tuple
+            results = results._asdict()
+        elif isinstance(results, cabc.Mapping):
+            pass
+        elif hasattr(results, "__dict__"):  # regular object
+            results = vars(results)
+        else:
+            raise ValueError(
+                "Expected results as mapping, named_tuple, object, "
+                f"got {type(results).__name__!r}: {results}\n  {self}"
+                f"\n  {debug_var_tip}"
+            )
+
+        fn_required = self._fn_provides
+        if fn_required:
+            renames = {get_keyword(i): i for i in fn_required}  # +1 useless key: None
+            renames.pop(None, None)
+            fn_expected = fn_required = [get_keyword(i) or i for i in fn_required]
+        else:
+            fn_expected = fn_required = renames = ()
+
+        if is_rescheduled:
+            # Canceled sfx(ed) are welcomed.
+            fn_expected = iset([*fn_expected, *(i for i in self.provides if is_sfx(i))])
+
+        res_names = results.keys()
+
+        ## Clip unknown outputs (handy for reuse).
+        #
+        unknown = [i for i in (res_names - fn_expected) if not is_pure_sfx(i)]
+        if unknown:
+            unknown = list(unknown)
+            log.warning(
+                "Results%s contained +%i unknown provides%s - will DELETE them!\n  %s",
+                list(res_names),
+                len(unknown),
+                list(unknown),
+                self,
+            )
+            # Filter results, don't mutate them.
+            # NOTE: too invasive when no-evictions!?
+            results = {k: v for k, v in results.items() if k not in unknown}
+
+        missmatched = fn_required - res_names
+        if missmatched:
+            if is_rescheduled:
+                log.warning(
+                    "... Op %r did not provide%s", self.name, list(missmatched),
+                )
+            else:
+                raise ValueError(
+                    f"Got x{len(results)} results({list(results)}) mismatched "
+                    f"-{len(missmatched)} provides({list(fn_expected)}):"
+                    f" {list(missmatched)}\n  {self}\n  {debug_var_tip}"
+                )
+
+        if renames:
+            results = {renames.get(k, k): v for k, v in results.items()}
+
+        return results
+
+    def _zip_results_plain(self, results, is_rescheduled) -> dict:
+        """Handle result sequence: no-result, single-item, or many. """
+        fn_expected = self._fn_provides
+        nexpected = len(fn_expected)
+
+        if results is NO_RESULT or results is NO_RESULT_BUT_SFX:
+            results = ()
+            ngot = 0
+
+        elif nexpected == 1:
+            results = [results]
+            ngot = 1
+
+        else:
+            # nexpected == 0 was method's 1st check.
+            assert nexpected > 1, nexpected
+            if isinstance(results, (str, bytes)) or not isinstance(
+                results, cabc.Iterable
+            ):
+                raise TypeError(
+                    f"Expected x{nexpected} ITERABLE results, "
+                    f"got {type(results).__name__!r}: {results}\n  {self}"
+                    f"\n  {debug_var_tip}"
+                )
+            ngot = len(results)
+
+        if ngot < nexpected and not is_rescheduled:
+            raise ValueError(
+                f"Got {ngot - nexpected} fewer results, while expected x{nexpected} "
+                f"provides({list(fn_expected)})!\n  {self}"
+                f"\n  {debug_var_tip}"
+            )
+
+        if ngot > nexpected:
+            ## Less problematic if not expecting anything but got something
+            #  (e.g reusing some function for sideffects).
+            extra_results_loglevel = logging.INFO if nexpected == 0 else logging.WARNING
+            logging.log(
+                extra_results_loglevel,
+                "Got +%s more results, while expected "
+                "x%s provides%s\n  results: %s\n  %s",
+                ngot - nexpected,
+                nexpected,
+                list(fn_expected),
+                results,
+                self,
+            )
+
+        return dict(zip(fn_expected, results))  # , fillvalue=UNSET))
+
     def _zip_results_with_provides(self, results) -> dict:
         """Zip results with expected "real" (without sideffects) `provides`."""
         from .config import is_reschedule_operations
 
-        fn_expected = self._fn_provides
-        rescheduled = first_solid(is_reschedule_operations(), self.rescheduled)
+        is_rescheduled = first_solid(is_reschedule_operations(), self.rescheduled)
         if self.returns_dict:
+            results = self._zip_results_returns_dict(results, is_rescheduled)
 
-            if hasattr(results, "_asdict"):  # named tuple
-                results = results._asdict()
-            elif isinstance(results, cabc.Mapping):
-                pass
-            elif hasattr(results, "__dict__"):  # regular object
-                results = vars(results)
-            else:
-                raise ValueError(
-                    "Expected results as mapping, named_tuple, object, "
-                    f"got {type(results).__name__!r}: {results}\n  {self}"
-                    f"\n  {debug_var_tip}"
-                )
-
-            fn_required = fn_expected
-            if rescheduled:
-                # Canceled sfx are welcomed.
-                fn_expected = iset(self.provides)
-
-            res_names = results.keys()
-
-            ## Allow unknown outs when dict,
-            #  bc we can safely ignore them (and it's handy for reuse).
-            #
-            unknown = res_names - fn_expected
-            if unknown:
-                unknown = list(unknown)
-                log.info(
-                    "Results%s contained +%s unknown provides%s\n  %s",
-                    list(res_names),
-                    len(unknown),
-                    list(unknown),
-                    self,
-                )
-
-            missmatched = fn_required - res_names
-            if missmatched:
-                if rescheduled:
-                    log.warning(
-                        "... Op %r did not provide%s",
-                        self.name,
-                        list(fn_expected - res_names),
-                    )
-                else:
-                    raise ValueError(
-                        f"Got x{len(results)} results({list(results)}) mismatched "
-                        f"-{len(missmatched)} provides({list(fn_expected)})!\n  {self}"
-                        f"\n  {debug_var_tip}"
-                    )
-
-        elif rescheduled and (results is NO_RESULT or results is NO_RESULT_BUT_SFX):
+        elif is_rescheduled and (results is NO_RESULT or results is NO_RESULT_BUT_SFX):
             results = (
                 {}
                 if results is NO_RESULT_BUT_SFX
@@ -719,7 +803,7 @@ class FnOp(Operation):
                 else {p: False for p in set(self.provides) if is_sfx(p)}
             )
 
-        elif not fn_expected:  # All provides were sideffects?
+        elif not self._fn_provides:  # All provides were sideffects?
             if (
                 results is not None
                 and results is not NO_RESULT
@@ -736,54 +820,7 @@ class FnOp(Operation):
             results = {}
 
         else:  # Handle result sequence: no-result, single-item, many
-            nexpected = len(fn_expected)
-
-            if results is NO_RESULT or results is NO_RESULT_BUT_SFX:
-                results = ()
-                ngot = 0
-
-            elif nexpected == 1:
-                results = [results]
-                ngot = 1
-
-            else:
-                # nexpected == 0 was method's 1st check.
-                assert nexpected > 1, nexpected
-                if isinstance(results, (str, bytes)) or not isinstance(
-                    results, cabc.Iterable
-                ):
-                    raise TypeError(
-                        f"Expected x{nexpected} ITERABLE results, "
-                        f"got {type(results).__name__!r}: {results}\n  {self}"
-                        f"\n  {debug_var_tip}"
-                    )
-                ngot = len(results)
-
-            if ngot < nexpected and not rescheduled:
-                raise ValueError(
-                    f"Got {ngot - nexpected} fewer results, while expected x{nexpected} "
-                    f"provides({list(fn_expected)})!\n  {self}"
-                    f"\n  {debug_var_tip}"
-                )
-
-            if ngot > nexpected:
-                ## Less problematic if not expecting anything but got something
-                #  (e.g reusing some function for sideffects).
-                extra_results_loglevel = (
-                    logging.INFO if nexpected == 0 else logging.WARNING
-                )
-                logging.log(
-                    extra_results_loglevel,
-                    "Got +%s more results, while expected "
-                    "x%s provides%s\n  results: %s\n  %s",
-                    ngot - nexpected,
-                    nexpected,
-                    list(fn_expected),
-                    results,
-                    self,
-                )
-
-            results = dict(zip(fn_expected, results))  # , fillvalue=UNSET))
+            results = self._zip_results_plain(results, is_rescheduled)
 
         assert isinstance(
             results, cabc.Mapping
@@ -798,6 +835,7 @@ class FnOp(Operation):
         return results
 
     def compute(self, named_inputs=None, outputs: Items = None) -> dict:
+        ok = False
         try:
             self.validate_fn_name()
             assert self.name is not None, self
@@ -820,23 +858,25 @@ class FnOp(Operation):
                     key: val for key, val in results_op.items() if key in outputs
                 }
 
+            ok = True
             return results_op
-        except Exception as ex:
-            jetsam(
-                ex,
-                locals(),
-                "outputs",
-                "aliases",
-                "results_fn",
-                "results_op",
-                operation="self",
-                args=lambda locs: {
-                    "positional": locs.get("positional"),
-                    "varargs": locs.get("varargs"),
-                    "kwargs": locs.get("kwargs"),
-                },
-            )
-            raise
+        finally:
+            if not ok:
+                ex = sys.exc_info()[1]
+                save_jetsam(
+                    ex,
+                    locals(),
+                    "outputs",
+                    "aliases",
+                    "results_fn",
+                    "results_op",
+                    operation="self",
+                    args=lambda locs: {
+                        "positional": locs.get("positional"),
+                        "varargs": locs.get("varargs"),
+                        "kwargs": locs.get("kwargs"),
+                    },
+                )
 
     def __call__(self, *args, **kwargs):
         """Like dict args, delegates to :meth:`.compute()`."""
@@ -875,7 +915,7 @@ def operation(
     marshalled=UNSET,
     returns_dict=UNSET,
     node_props: Mapping = UNSET,
-):
+) -> FnOp:
     r"""
     An :term:`operation` factory that works like a "fancy decorator".
 
@@ -933,6 +973,13 @@ def operation(
         <returns dictionary>`, in which case must return a dictionary that containing
         (at least) those named elements.
 
+
+        .. provides-note-start
+        .. Note::
+            When joining a pipeline this must not be empty, or will scream!
+            (an operation without provides would always be pruned)
+        .. provides-note-end
+
         .. seealso::
             - :term:`provides`
             - :term:`modifier`
@@ -940,6 +987,7 @@ def operation(
             - :attr:`.FnOp.op_provides`
             - :attr:`.FnOp._fn_provides`
 
+
     :param aliases:
         an optional mapping of `provides` to additional ones
     :param rescheduled:
