@@ -1,7 +1,7 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """
-:term:`modifier`\\s change :term:`dependency` behavior during :term:`planning` & :term:`execution`.
+:term:`modifier`\\s (print-out with :term:`diacritic`\\s) change :term:`dependency` behavior during :term:`planning` & :term:`execution`.
 
 The `needs` and `provides` annotated with *modifiers* designate, for instance,
 :term:`optional <optionals>` function arguments, or "ghost" :term:`sideffects`.
@@ -171,7 +171,7 @@ class _Modifier(str):
     #: pre-calculated representation
     _repr: str
     #: The name of a modifier function here, needed to reconstruct
-    #: cstor code in :attr:`cmd`
+    #: cstor code in :meth:`.cmd()`
     _func: str
     #: Map my name in `needs` into this kw-argument of the function.
     #: :func:`get_keyword()` returns it.
@@ -192,7 +192,7 @@ class _Modifier(str):
     #: - If it is an empty tuple`, it is an abstract sideffect,
     #:    and :func:`is_pure_optional()` returns True.
     #: - If not empty :func:`is_sfxed()` returns true
-    #:   (the :attr:`sideffected`).
+    #:   (the :attr:`._sideffected`).
     _sfx_list: Tuple[Union[str, None]] = ()
 
     def __new__(
@@ -304,8 +304,9 @@ def _modifier(
         a falsy (but not ``None``) value, to disable the automatic interpeting of
         the dependency name as a json pointer path, regardless of any containing slashes.
     :param kw:
-        Not used here, any given kKVs are assigned as :class:`_Modifier` attributes,
-        for client code to extend its own modifiers.
+        extra kv pairs assigned as :class:`_Modifier` **private** attributes
+        (prefixed with ``_`` if not already) only if values are not null.
+        Here used for :term:`implicit`, and client code may extend its own modifiers.
     """
     if jsonp is not None:
         kw["_jsonp"] = jsonp  # WARN: must be False or a collection for jsonp-accessors!
@@ -321,8 +322,13 @@ def _modifier(
 
     args = (name, keyword, optional, accessor, sideffected, sfx_list)
     formats = _match_modifier_args(*args)
+
+    ## Private-ize all keywords and throw away nulls.
+    kw = {
+        k if k.startswith("_") else f"_{k}": v for k, v in kw.items() if v is not None
+    }
     if not formats:
-        if kw.get("_jsonp") is not None:
+        if kw:
             # Just jsonp given.
             assert not accessor and (optional, accessor, sideffected, sfx_list) == (
                 None,
@@ -330,7 +336,7 @@ def _modifier(
                 None,
                 (),
             ), locals()
-            return _Modifier(name, name, "jsonp", *args[1:], jsonp=jsonp)
+            return _Modifier(name, name, "modify", *args[1:], **kw)
 
         # Make a plain string instead.
         return str(name)
@@ -388,14 +394,88 @@ def modifier_withset(
     return _modifier(name=name, **kw)
 
 
+def modify(name: str, jsonp=None, implicit=None) -> _Modifier:
+    """
+    Generic :term:`modifier` for term:`json pointer path` & :term:`implicit` dependencies.
+
+    :param jsonp:
+        If given, it may be the pre-splitted *parts* of the json pointer path
+        for the dependency -- in that case, the dependency name is irrelevant -- or
+        a falsy (but not ``None``) value, to disable the automatic interpeting of
+        the dependency name as a json pointer path, regardless of any containing slashes.
+
+        .. Tip::
+            If accessing pandas, you may pass an already splitted path with
+            its last *part* being a `callable indexer
+            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
+    :param implicit:
+        :term:`implicit` dependencies are not fed into/out of the function.
+
+    **Example:**
+
+    Let's use *json pointer dependencies* along with the default :term:`conveyor operation`
+    to build an operation copying values around in the solution:
+
+        >>> from graphtik import operation, compose, modify
+
+        >>> copy_values = operation(
+        ...     fn=None,  # ask for the "conveyor op"
+        ...     name="copy a+b-->A+BB",
+        ...     needs=["inputs/a", "inputs/b"],
+        ...     provides=["RESULTS/A", "RESULTS/BB"]
+        ... )
+
+        >>> results = copy_values.compute({"inputs": {"a": 1, "b": 2}})
+        Traceback (most recent call last):
+        ValueError: Failed matching inputs <=> needs for FnOp(name='copy a+b-->A+BB',
+            needs=['inputs/a'($), 'inputs/b'($)],
+            provides=['RESULTS/A'($), 'RESULTS/BB'($)],
+            fn='identity_fn'):
+            1. Missing compulsory needs['inputs/a'($), 'inputs/b'($)]!
+            +++inputs: ['inputs']
+
+        >>> results = copy_values.compute({"inputs/a": 1, "inputs/b": 2})
+        >>> results
+        {'RESULTS/A'($): 1, 'RESULTS/BB'($): 2}
+
+    Notice that the :term:`hierarchical dependencies <subdoc>` did not yet worked,
+    because *jsonp* modifiers work internally with :term:`accessor`\\s, and
+    :class:`.FnOp` is unaware of them -- it's the :class:`.Solution`
+    class that supports *accessors**, and this requires the operation to be wrapped
+    in a pipeline (see below).
+
+    Note also that it we see the "representation' of the key as ``'RESULTS/A'($)``
+    but the actual string value simpler:
+
+        >>> str(next(iter(results)))
+        'RESULTS/A'
+
+    The results were not nested, because this modifer works with :term:`accessor` functions,
+    that act only on a real :class:`.Solution`, given to the operation only when wrapped
+    in a pipeline (as done below).
+
+    Now watch how these paths access deep into solution when the same operation
+    is wrapped in a pipeline:
+
+        >>> pipe = compose("copy pipe", copy_values)
+        >>> sol = pipe.compute({"inputs": {"a": 1, "b": 2}}, outputs="RESULTS")
+        >>> sol
+        {'RESULTS': {'A': 1, 'BB': 2}}
+
+    .. graphtik::
+    """
+    return _modifier(name, jsonp=jsonp, implicit=implicit)
+
+
 def keyword(
     name: str, keyword: str = None, accessor: Accessor = None, jsonp=None
 ) -> _Modifier:
     """
-    Annotate a :term:`needs` that (optionally) maps `inputs` name --> *keyword* argument name.
+    Annotate a :term:`dependency` that maps to a different name in the underlying function.
 
-    The value of a *keyword* dependency is passed in as *keyword argument*
-    to the underlying function.
+    - The value of a *keyword* `needs` dependency is passed in as *keyword argument*
+      to the underlying function.
+    - For *keyword* to work in `provides`, the operation must be a :term:`returns dictionary`.
 
     :param keyword:
         The argument-name corresponding to this named-input.
@@ -428,30 +508,50 @@ def keyword(
 
     **Example:**
 
-    In case the name of the function arguments is different from the name in the
-    `inputs` (or just because the name in the `inputs` is not a valid argument-name),
+    In case the name of a function input argument is different from the name in the
+    :term:`graph` (or just because the name in the `inputs` is not a valid argument-name),
     you may *map* it with the 2nd argument of :func:`.keyword`:
 
         >>> from graphtik import operation, compose, keyword
 
-        >>> @operation(needs=['a', keyword("name-in-inputs", "b")], provides="sum")
-        ... def myadd(a, *, b):
-        ...    return a + b
-        >>> myadd
-        FnOp(name='myadd',
-                            needs=['a', 'name-in-inputs'(>'b')],
-                            provides=['sum'],
-                            fn='myadd')
+        >>> @operation(needs=[keyword("name-in-inputs", "fn_name")], provides="result")
+        ... def foo(*, fn_name):  # it works also with non-positional args
+        ...    return fn_name
+        >>> foo
+        FnOp(name='foo',
+             needs=['name-in-inputs'(>'fn_name')],
+             provides=['result'],
+             fn='foo')
 
-        >>> graph = compose('mygraph', myadd)
-        >>> graph
-        Pipeline('mygraph', needs=['a', 'name-in-inputs'], provides=['sum'], x1 ops: myadd)
+        >>> pipe = compose('map a need', foo)
+        >>> pipe
+        Pipeline('map a need', needs=['name-in-inputs'], provides=['result'], x1 ops: foo)
 
-        >>> sol = graph.compute({"a": 5, "name-in-inputs": 4})['sum']
-        >>> sol
-        9
+        >>> sol = pipe.compute({"name-in-inputs": 4})
+        >>> sol['result']
+        4
 
         .. graphtik::
+
+    You can do the same thing to the results of a :term:`returns dictionary` operation:
+
+        >>> op = operation(lambda: {"fn key": 1},
+        ...                name="renaming `provides` with a `keyword`",
+        ...                provides=keyword("graph key", "fn key"),
+        ...                returns_dict=True)
+        >>> op
+        FnOp(name='renaming `provides` with a `keyword`',
+             provides=['graph key'(>'fn key')],
+             fn{}='<lambda>')
+
+    .. graphtik::
+
+    .. hint::
+        Mapping `provides` names wouldn't make sense for regular operations, since
+        these are defined arbitrarily at the operation level.
+        OTOH, the result names of :term:`returns dictionary` operation are decided
+        by the underlying function, which may lie beyond the control of the user
+        (e.g. from a 3rd-party object).
     """
     # Must pass a truthy `keyword` bc cstor cannot not know its keyword.
     return _modifier(name, keyword=keyword or name, accessor=accessor, jsonp=jsonp)
@@ -555,7 +655,7 @@ def accessor(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
             <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
 
 
-    - Probably not very usefull -- see the :func:`jsonp` modifier for an integrated
+    - Probably not very usefull -- see the :func:`.modify` modifier for an integrated
       use case.
     - To combine it with `optional`, `keyword`, etc use the rest modifier factories
       and pass an argument value to their `accessor` parameter.
@@ -563,79 +663,6 @@ def accessor(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
     return _modifier(name, accessor=accessor, jsonp=jsonp)
 
 
-def jsonp(name: str, jsonp=None) -> _Modifier:
-    """
-    Control the automatic interpretation of dependencies containing slashes into :term:`json pointer path`.
-
-    :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-
-
-    **Example:**
-
-    Let's use *json pointer dependencies* along with the default :term:`conveyor operation`
-    to build an operation copying values around in the solution:
-
-        >>> from graphtik import operation, compose, jsonp
-
-        >>> copy_values = operation(
-        ...     fn=None,  # ask for the "conveyor op"
-        ...     name="copy a+b-->A+BB",
-        ...     needs=["inputs/a", "inputs/b"],
-        ...     provides=["RESULTS/A", "RESULTS/BB"]
-        ... )
-
-        >>> results = copy_values.compute({"inputs": {"a": 1, "b": 2}})
-        Traceback (most recent call last):
-        ValueError: Failed preparing needs:
-            1. Missing compulsory needs['inputs/a'($), 'inputs/b'($)]!
-            +++inputs: ['inputs']
-            +++FnOp(name='copy a+b-->A+BB',
-                                   needs=['inputs/a'($), 'inputs/b'($)],
-                                   provides=['RESULTS/A'($), 'RESULTS/BB'($)],
-                                   fn='identity_fn')
-
-        >>> results = copy_values.compute({"inputs/a": 1, "inputs/b": 2})
-        >>> results
-        {'RESULTS/A'($): 1, 'RESULTS/BB'($): 2}
-
-    Notice that the :term:`hierarchical dependencies <subdoc>` did not yet worked,
-    because *jsonp* modifiers work internally with :term:`accessor`\\s, and
-    :class:`.FnOp` is unaware of them -- it's the :class:`.Solution`
-    class that supports *accessors**, and this requires the operation to be wrapped
-    in a pipeline (see below).
-
-    Note also that it we see the "representation' of the key as ``'RESULTS/A'($)``
-    but the actual string value simpler:
-
-        >>> str(next(iter(results)))
-        'RESULTS/A'
-
-    The results were not nested, because this modifer works with :term:`accessor` functions,
-    that act only on a real :class:`.Solution`, given to the operation only when wrapped
-    in a pipeline (as done below).
-
-    Now watch how these paths access deep into solution when the same operation
-    is wrapped in a pipeline:
-
-        >>> pipe = compose("copy pipe", copy_values)
-        >>> sol = pipe.compute({"inputs": {"a": 1, "b": 2}}, outputs="RESULTS")
-        >>> sol
-        {'RESULTS': {'A': 1, 'BB': 2}}
-
-    .. graphtik::
-    """
-    return _modifier(name, jsonp=jsonp)
-
-
 def vararg(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
     """
     Annotate a :term:`varargish` `needs` to  be fed as function's ``*args``.
@@ -745,11 +772,10 @@ def varargs(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
         {'a': 5, 'sum': [5]}
         >>> graph(a=5, b=0xBAD)
         Traceback (most recent call last):
-        ...
-        ValueError: Failed preparing needs:
-            1. Expected needs['b'(+)] to be non-str iterables!
+        ValueError: Failed matching inputs <=> needs for FnOp(name='enlist',
+                needs=['a', 'b'(+)], provides=['sum'], fn='enlist'):
+            1. Expected varargs inputs to be non-str iterables: {'b'(+): 2989}
             +++inputs: ['a', 'b']
-            +++FnOp(name='enlist', needs=['a', 'b'(+)], provides=['sum'], fn='enlist')
 
     .. varargs-mistake-start
     .. Attention::
@@ -758,14 +784,12 @@ def varargs(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
 
         >>> graph(a=5, b="mistake")
         Traceback (most recent call last):
-        ...
-        ValueError: Failed preparing needs:
-            1. Expected needs['b'(+)] to be non-str iterables!
+        ValueError: Failed matching inputs <=> needs for FnOp(name='enlist',
+                    needs=['a', 'b'(+)],
+                    provides=['sum'],
+                    fn='enlist'):
+            1. Expected varargs inputs to be non-str iterables: {'b'(+): 'mistake'}
             +++inputs: ['a', 'b']
-            +++FnOp(name='enlist',
-                                   needs=['a', 'b'(+)],
-                                   provides=['sum'],
-                                   fn='enlist')
 
     .. varargs-mistake-end
 
@@ -860,10 +884,18 @@ def sfxed(
     optional: bool = None,
     accessor: Accessor = None,
     jsonp=None,
+    implicit=None,
 ) -> _Modifier:
     r"""
     Annotates a :term:`sideffected` dependency in the solution sustaining side-effects.
 
+    :param dependency:
+        the actual dependency receiving the sideffect, which will be fed into/out
+        of the function (unless marked as :term:`implicit`).
+    :param sfx0:
+        the 1st (arbitrary object) sideffect marked as "acting" on the `dependency`.
+    :param sfx0:
+        more (arbitrary object) sideffects (like the `sfx0`)
     :param keyword:
         the name for the function argument it corresponds.
         When optional, it becomes the same as `name` if falsy, so as
@@ -882,7 +914,8 @@ def sfxed(
             If accessing pandas, you may pass an already splitted path with
             its last *part* being a `callable indexer
             <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-
+    :param implicit:
+        :term:`implicit` dependencies are not fed into/out of the function.
 
     Like :func:`.sfx` but annotating a *real* :term:`dependency` in the solution,
     allowing that dependency to be present both in :term:`needs` and :term:`provides`
@@ -974,6 +1007,7 @@ def sfxed(
         sfx_list=(sfx0, *sfx_list),
         accessor=accessor,
         jsonp=jsonp,
+        implicit=implicit,
     )
 
 
@@ -1012,7 +1046,7 @@ def get_keyword(dep) -> Optional[str]:
     All non-varargish optionals are "keyword" (including sideffected ones).
 
     :return:
-        the :attr:`.keyword`
+        the :attr:`._keyword`
     """
     return getattr(dep, "_keyword", None)
 
@@ -1024,7 +1058,7 @@ def is_optional(dep) -> Optional[_Optionals]:
     Varargish & optional sideffects are included.
 
     :return:
-        the :attr:`.optional`
+        the :attr:`._optional`
     """
     return getattr(dep, "_optional", None)
 
@@ -1054,7 +1088,7 @@ def is_sfx(dep) -> Optional[str]:
     Check if a dependency is :term:`sideffects` or :term:`sideffected`.
 
     :return:
-        the :attr:`.sideffected`
+        the :attr:`._sideffected`
     """
     return getattr(dep, "_sideffected", None)
 
@@ -1065,16 +1099,26 @@ def is_pure_sfx(dep) -> bool:
 
 
 def is_sfxed(dep) -> bool:
-    """Check if it is :term:`sideffected`."""
+    """
+    Check if it is :term:`sideffected`.
+
+    :return:
+        the :attr:`._sfx_list` if it is  a *sideffected* dep, None/empty-tuple otherwise
+    """
     return getattr(dep, "_sideffected", None) and getattr(dep, "_sfx_list", None)
 
 
+def is_implicit(dep) -> bool:
+    """Return if it is a :term:`implicit` dependency. """
+    return getattr(dep, "_implicit", None)
+
+
 def get_accessor(dep) -> bool:
     """
     Check if dependency has an :term:`accessor`, and get it (if funcs below are unfit)
 
     :return:
-        the :attr:`accessor`
+        the :attr:`._accessor`
     """
     return getattr(dep, "_accessor", None)
 
@@ -1117,7 +1161,7 @@ def dependency(dep) -> str:
 
     For non-sideffects, it coincides with str(), otherwise,
     the the pure-sideffect string or the existing :term:`sideffected` dependency
-    stored in :attr:`sideffected`.
+    stored in :attr:`._sideffected`.
     """
     return str(dep) if is_sfx(dep) else dep._sideffected
 
@@ -1151,18 +1195,17 @@ def dep_renamed(dep, ren) -> Union[_Modifier, str]:
 
 def dep_singularized(dep) -> Iterable[Union[str, _Modifier]]:
     """
-    Yield one sideffected for each sfx in :attr:`.sfx_list`, or iterate `dep` in other cases.
+    Return one sideffected for each sfx in :attr:`._sfx_list`, or iterate `dep` in other cases.
     """
+    sfx_list = is_sfxed(dep)
     return (
-        (modifier_withset(dep, sfx_list=(s,)) for s in dep._sfx_list)
-        if is_sfxed(dep)
-        else (dep,)
+        [modifier_withset(dep, sfx_list=(s,)) for s in sfx_list] if sfx_list else (dep,)
     )
 
 
 def dep_stripped(dep) -> Union[str, _Modifier]:
     """
-    Return the :attr:`_Modifier.sideffected` if `dep` is :term:`sideffected`, `dep` otherwise,
+    Return the :attr:`._sideffected` if `dep` is :term:`sideffected`, `dep` otherwise,
 
     conveying all other properties of the original modifier to the stripped dependency.
     """
