@@ -9,8 +9,10 @@
     (<5ms on a 2019 fast PC)
 """
 
+import inspect
 import logging
 import re
+import sys
 from collections import abc as cabc
 from typing import Callable, List, Mapping, Union
 
@@ -24,8 +26,8 @@ from .base import (
     Plottable,
     RenArgs,
     aslist,
-    jetsam,
 )
+from .jetsam import save_jetsam
 from .modifier import dep_renamed
 
 log = logging.getLogger(__name__)
@@ -38,6 +40,9 @@ class NULL_OP(Operation):
     :seealso: :ref:`operation-merging`
     """
 
+    #: Dummy `provides` for pipeline's "last minute" check not to scream.
+    provides = (None,)
+
     def __init__(self, name):
         self.name = name
 
@@ -104,12 +109,13 @@ def build_network(
                 op_kw["renamer"] = parent_wrapper
             op = op.withset(**op_kw)
 
-        ## Last minute checks
-        #  (or Plot will fail later).
+        ## Last minute checks, couldn't check earlier due to builder pattern.
         #
         if hasattr(op, "fn"):
-            ## Could not check earlier due to builder pattern.
             op.validate_fn_name()
+        if not getattr(op, "op_provides", op.provides):
+            TypeError(f"`provides` must not be empty!")
+
         return op
 
     merge_set = iset()  # Preseve given node order.
@@ -173,8 +179,12 @@ class Pipeline(Operation):
 
         ## Set data asap, for debugging, although `net.withset()` will reset them.
         self.name = name
-        # Remember Outputs for future `compute()`?
+        #: Fake function attributes.
+        self.__name__ = self.__qualname__ = name
+
+        #: Remember `outputs` for future `compute()`?
         self.outputs = outputs
+        #: Remember `predicate` for future `compute()`?
         self.predicate = predicate
 
         # Prune network
@@ -233,10 +243,12 @@ class Pipeline(Operation):
             the name for the new pipeline:
 
             - if `None`, the same name is kept;
-            - if True, a distinct name is  devised::
+            - if True, a distinct name is devised::
 
                 <old-name>-<uid>
 
+            - if ellipses(``...``), the name of the function where this function
+              call happened is used,
             - otherwise, the given `name` is applied.
         :param rescheduled:
             applies :term:`reschedule`\\d to all contained `operations`
@@ -337,21 +349,14 @@ class Pipeline(Operation):
             the :term:`execution plan` satisfying the given `inputs`, `outputs` & `predicate`
 
         :raises ValueError:
-            - If `outputs` asked do not exist in network, with msg:
-
-                *Unknown output nodes: ...*
-
-            - If solution does not contain any operations, with msg:
-
-                *Unsolvable graph: ...*
-
-            - If given `inputs` mismatched plan's :attr:`needs`, with msg:
-
-                *Plan needs more inputs...*
-
-            - If net cannot produce asked `outputs`, with msg:
-
-                *Unreachable outputs...*
+            *Unknown output nodes...*
+                if `outputs` asked do not exist in network.
+            *Unsolvable graph: ...*
+                if it cannot produce any `outputs` from the given `inputs`.
+            *Plan needs more inputs...*
+                if given `inputs` mismatched plan's :attr:`needs`.
+            *Unreachable outputs...*
+                if net cannot produce asked `outputs`.
         """
         outputs = self.outputs if outputs == UNSET else outputs
         predicate = self.predicate if predicate == UNSET else predicate
@@ -367,7 +372,7 @@ class Pipeline(Operation):
         layered_solution=None,
     ) -> "Solution":
         """
-        Compile a plan & :term:`execute` the graph, sequentially or parallel.
+        Compile & :term:`execute` the plan, log :term:`jetsam` & plot :term:`plottable` on errors.
 
         .. Attention::
             If intermediate :term:`planning` is successful, the "global
@@ -420,6 +425,7 @@ class Pipeline(Operation):
         """
         from .config import reset_abort
 
+        ok = False
         try:
             if named_inputs is UNSET:
                 named_inputs = {}
@@ -429,7 +435,7 @@ class Pipeline(Operation):
             if not predicate:
                 predicate = self.predicate
 
-            log.debug("=== Compiling %s...", self)
+            log.info("=== Compiling pipeline(%s) ...", self.name)
             # # Discover nested inputs given.
             # plan = net.compile(
             #     self._nested_inputs_given(named_inputs), outputs, predicate
@@ -447,10 +453,33 @@ class Pipeline(Operation):
                 layered_solution=layered_solution,
             )
 
+            ok = True
             return solution
-        except Exception as ex:
-            jetsam(ex, locals(), "plan", "solution", "outputs", network="net")
-            raise
+        finally:
+            if not ok:
+                ex = sys.exc_info()[1]
+                jetsam = save_jetsam(
+                    ex,
+                    locals(),
+                    "plan",
+                    "solution",
+                    "outputs",
+                    pipeline="self",
+                    network="net",
+                )
+
+                try:
+                    jetsam.log_n_plot()
+                except Exception as ex2:
+                    log.warning(
+                        "Suppressed error while logging/plotting jetsam of %s: %s(%s)"
+                        "\n  +--annotations:%s",
+                        self,
+                        jetsam,
+                        type(ex).__name__,
+                        ex,
+                        exc_info=True,
+                    )
 
     def __call__(self, **input_kwargs) -> "Solution":
         """
@@ -475,9 +504,9 @@ def nest_any_node(ren_args: RenArgs) -> str:
 
 
 def compose(
-    name,
-    op1,
-    *operations,
+    name: Union[str, type(...), None],
+    op1: Operation,
+    *operations: Operation,
     outputs: Items = None,
     rescheduled=None,
     endured=None,
@@ -494,8 +523,10 @@ def compose(
     Operations given earlier (further to the left) override those following
     (further to the right), similar to `set` behavior (and contrary to `dict`).
 
-    :param str name:
-        A optional name for the graph being composed by this object.
+    :param name:
+        An optional name for the graph being composed by this object.
+        If ellipses(``...``), derrived from function name where the pipeline
+        is defined.
     :param op1:
         syntactically force at least 1 operation
     :param operations:
@@ -571,6 +602,12 @@ def compose(
         - If `nest` callable/dictionary produced an non-string or empty name
           (see (NetworkPipeline))
     """
+    if name is ...:
+        pf = inspect.currentframe().f_back
+        if pf:
+            name = pf.f_code.co_name
+        del pf
+
     operations = (op1,) + operations
     if not all(isinstance(op, Operation) for op in operations):
         bad_ops = [op for op in operations if not isinstance(op, Operation)]
