@@ -4,6 +4,7 @@
 :term:`compose` :term:`network` of operations & dependencies, :term:`compile` the :term:`plan`.
 """
 import logging
+import sys
 from collections import abc, defaultdict
 from functools import partial
 from itertools import count
@@ -24,22 +25,25 @@ import networkx as nx
 from boltons.iterutils import pairwise
 from boltons.setutils import IndexedSet as iset
 
-from .base import Items, Operation, PlotArgs, Plottable, astuple, jetsam
+from .base import Items, Operation, PlotArgs, Plottable, astuple
 from .config import is_debug, is_skip_evictions
+from .jetsam import save_jetsam
 from .modifier import (
-    modifier_withset,
     dep_renamed,
     dep_stripped,
-    get_keyword,
     get_jsonp,
-    jsonp,
+    get_keyword,
+    is_implicit,
     is_optional,
     is_pure_sfx,
     is_sfx,
+    modifier_withset,
+    modify,
     optional,
 )
 
 NodePredicate = Callable[[Any, Mapping], bool]
+OpMap = Mapping[Operation, Any]
 
 #: If this logger is *eventually* DEBUG-enabled,
 #: the string-representation of network-objects (network, plan, solution)
@@ -204,7 +208,40 @@ def clone_graph_with_stripped_sfxed(graph):
     return clone
 
 
-def unsatisfied_operations(dag, inputs: Iterable) -> List:
+def _topo_sort_nodes(dag) -> iset:
+    """
+    Topo-sort dag by execution order & operation-insertion order to break ties.
+
+    This means (probably!?) that the first inserted win the `needs`, but
+    the last one win the `provides` (and the final solution).
+
+    Inform user in case of cycles.
+    """
+    node_keys = dict(zip(dag.nodes, count()))
+    try:
+        return iset(nx.lexicographical_topological_sort(dag, key=node_keys.get))
+    except nx.NetworkXUnfeasible as ex:
+        import sys
+        from textwrap import dedent
+
+        tb = sys.exc_info()[2]
+        msg = dedent(
+            f"""
+            {ex}
+
+            TIP:
+                Launch a post-mortem debugger, move 3 frames UP, and
+                plot the `graphtik.planning.Network' class in `self`
+                to discover the cycle.
+
+                If GRAPHTIK_DEBUG enabled, this plot will be stored tmp-folder
+                automatically :-)
+            """
+        )
+        raise nx.NetworkXUnfeasible(msg).with_traceback(tb)
+
+
+def unsatisfied_operations(dag, inputs: Iterable) -> Tuple[OpMap, iset]:
     """
     Traverse topologically sorted dag to collect un-satisfied operations.
 
@@ -223,7 +260,7 @@ def unsatisfied_operations(dag, inputs: Iterable) -> List:
     :param inputs:
         an iterable of the names of the input values
     :return:
-        a list of unsatisfied operations to prune
+        a 2-tuple with ({pruned-op, unsatisfied-explanation}, topo-sorted-nodes)
 
     """
     # Collect data that will be produced.
@@ -233,26 +270,32 @@ def unsatisfied_operations(dag, inputs: Iterable) -> List:
     # To collect the map of operations --> satisfied-needs.
     op_satisfaction = defaultdict(set)
     # To collect the operations to drop.
-    unsatisfied = []
-    # Topo-sort dag respecting operation-insertion order to break ties.
-    sorted_nodes = nx.topological_sort(dag)
-    for node in sorted_nodes:
+    pruned_ops = {}
+    ## Topo-sort dag respecting operation-insertion order to break ties.
+    sorted_nodes = _topo_sort_nodes(dag)
+
+    if log.isEnabledFor(logging.DEBUG):
+        log.debug("Topo-sorted nodes: %s", list(yield_node_names(sorted_nodes)))
+    for i, node in enumerate(sorted_nodes):
         if isinstance(node, Operation):
             if not dag.adj[node]:
-                # Prune operations that ended up providing no output.
-                unsatisfied.append(node)
+                pruned_ops[node] = "needless-outputs"
+                log.info("... pruned step #%i due to needless-outputs\n  %s", i, node)
             else:
                 real_needs = set(
                     n for n, _, opt in dag.in_edges(node, data="optional") if not opt
                 )
-                # ## Sanity check that op's needs are never broken
+                satisfied_needs = op_satisfaction[node]
+                ## Sanity check that op's needs are never broken
                 # assert real_needs == set(n for n in node.needs if not is_optional(n))
-                if real_needs.issubset(op_satisfaction[node]):
+                if real_needs.issubset(satisfied_needs):
                     # Op is satisfied; mark its outputs as ok.
                     ok_data.update(yield_chaindocs(dag, dag.adj[node], ok_data))
                 else:
-                    # Prune operations with partial inputs.
-                    unsatisfied.append(node)
+                    pruned_ops[
+                        node
+                    ] = msg = f"unsatisfied-needs{list(real_needs - satisfied_needs)}"
+                    log.info("... pruned step #%i due to %s\n  %s", i, msg, node)
         elif isinstance(node, str):  # `str` are givens
             if node in ok_data:
                 # mark satisfied-needs on all future operations
@@ -261,7 +304,7 @@ def unsatisfied_operations(dag, inputs: Iterable) -> List:
         else:
             raise AssertionError(f"Unrecognized network graph node {node}")
 
-    return unsatisfied
+    return pruned_ops, sorted_nodes
 
 
 class Network(Plottable):
@@ -375,7 +418,7 @@ class Network(Plottable):
         def append_subdoc_chain(doc_parts):
             doc_chain = list(doc_parts)
             doc_chain = [
-                jsonp("/".join(doc_chain[: i + 1])) for i in range(len(doc_chain))
+                modify("/".join(doc_chain[: i + 1])) for i in range(len(doc_chain))
             ]
             # FIXME: subdocs ignore double-slashes or final slash!
             doc_chain = [p for p in doc_chain if p]
@@ -390,11 +433,13 @@ class Network(Plottable):
             if json_path:
                 append_subdoc_chain(json_path)
 
-            nkw, ekw = {}, {}
+            nkw, ekw = {}, {}  # node, edge props
             if is_optional(n):
                 ekw["optional"] = True
             if is_sfx(n):
                 ekw["sideffect"] = nkw["sideffect"] = True
+            if is_implicit(n):
+                ekw["implicit"] = True
             keyword = get_keyword(n)
             if keyword:
                 ekw["keyword"] = keyword
@@ -421,6 +466,8 @@ class Network(Plottable):
             if is_sfx(n):
                 kw["sideffect"] = True
                 graph.add_node(n, sideffect=True)
+            if is_implicit(n):
+                kw["implicit"] = True
 
             if n in alias_sources:
                 src_provide = alias_sources[n]
@@ -428,17 +475,6 @@ class Network(Plottable):
 
             graph.add_edge(operation, n, **kw)
 
-    def _topo_sort_nodes(self, dag) -> List:
-        """
-        Topo-sort dag by execution order, then by operation-insertion order
-        to break ties.
-
-        This means (probably!?) that the first inserted win the `needs`, but
-        the last one win the `provides` (and the final solution).
-        """
-        node_keys = dict(zip(dag.nodes, count()))
-        return nx.lexicographical_topological_sort(dag, key=node_keys.get)
-
     def _apply_graph_predicate(self, graph, predicate):
         to_del = []
         for node, data in graph.nodes.items():
@@ -449,12 +485,12 @@ class Network(Plottable):
                 raise ValueError(
                     f"Node-predicate({predicate}) failed due to: {ex}\n  node: {node}, {self}"
                 ) from ex
-        log.info("... predicate filtered out %s.", [op.name for op in to_del])
+        log.info("... predicate filtered out %s.", list(yield_node_names(to_del)))
         graph.remove_nodes_from(to_del)
 
     def _prune_graph(
         self, inputs: Items, outputs: Items, predicate: NodePredicate = None
-    ) -> Tuple[nx.DiGraph, Collection, Collection, Collection]:
+    ) -> Tuple[nx.DiGraph, Tuple, Tuple, OpMap]:
         """
         Determines what graph steps need to run to get to the requested
         outputs from the provided inputs:
@@ -473,9 +509,13 @@ class Network(Plottable):
             that should return true for nodes to include; if None, all nodes included.
 
         :return:
-            a 3-tuple with the *pruned_dag* & the needs/provides resolved based
-            on the given inputs/outputs
-            (which might be a subset of all needs/outputs of the returned graph).
+            a 4-tuple:
+
+            - the *pruned* :term:`execution dag`,
+            - net's needs & outputs based on the given inputs/outputs and the net
+              (may overlap, see :func:`collect_requirements()`),
+            - an {op, prune-explanation} dictionary
+
 
             Use the returned `needs/provides` to build a new plan.
 
@@ -484,7 +524,7 @@ class Network(Plottable):
 
                 *Unknown output nodes: ...*
         """
-        # TODO: break cycles here.
+        # TODO: break cycles based on weights here.
         dag = self.graph
 
         ##  When `inputs` is None, we have to keep all possible input nodes
@@ -544,6 +584,8 @@ class Network(Plottable):
                     )
                 )
 
+        comments: OpMap = {}
+
         # Drop stray input values and operations (if any).
         if outputs is not None:
             ## If caller requested specific outputs, we can prune any
@@ -556,22 +598,22 @@ class Network(Plottable):
                 ending_in_outputs.add(out)
             # Clone it, to modify it, or BUG@@ much later (e.g in eviction planing).
             broken_dag = broken_dag.subgraph(ending_in_outputs).copy()
-            if log.isEnabledFor(logging.INFO) and len(
-                list(yield_ops(ending_in_outputs))
-            ) != len(self.ops):
+
+            irrelevant_ops = [
+                op for op in yield_ops(dag) if op not in ending_in_outputs
+            ]
+            if irrelevant_ops:
+                comments.update((op, "outputs-irrelevant") for op in irrelevant_ops)
                 log.info(
-                    "... dropping output-irrelevant ops%s.",
-                    [
-                        op.name
-                        for op in dag
-                        if isinstance(op, Operation) and op not in ending_in_outputs
-                    ],
+                    "... dropping output-irrelevant ops%s.\n    +--outputs: %s",
+                    irrelevant_ops,
+                    outputs,
                 )
 
         # Prune unsatisfied operations (those with partial inputs or no outputs).
-        unsatisfied = unsatisfied_operations(broken_dag, satisfied_inputs)
-        if log.isEnabledFor(logging.INFO) and unsatisfied:
-            log.info("... dropping unsatisfied ops%s.", [op.name for op in unsatisfied])
+        unsatisfied, sorted_nodes = unsatisfied_operations(broken_dag, satisfied_inputs)
+        comments.update(unsatisfied)
+
         # Clone it, to modify it.
         pruned_dag = dag.subgraph(broken_dag.nodes - unsatisfied).copy()
         ## Clean unlinked data-nodes (except those both given & asked).
@@ -598,16 +640,19 @@ class Network(Plottable):
         assert inputs is not None or isinstance(inputs, abc.Collection)
         assert outputs is not None or isinstance(outputs, abc.Collection)
 
-        return pruned_dag, tuple(inputs), tuple(outputs)
+        return pruned_dag, sorted_nodes, tuple(inputs), tuple(outputs), comments
 
     def _build_execution_steps(
-        self, pruned_dag, inputs: Collection, outputs: Collection
+        self, pruned_dag, sorted_nodes, inputs: Collection, outputs: Collection
     ) -> List:
         """
         Create the list of operations and :term:`eviction` steps, to execute given IOs.
 
         :param pruned_dag:
             The original dag, pruned; not broken.
+        :param sorted_nodes:
+            an :class:`~boltons:boltons.setutils.IndexedSet` with all graph nodes
+            topo-sorted (including pruned ones)
         :param inputs:
             Not used(!), useless inputs will be evicted when the solution is created.
         :param outputs:
@@ -640,13 +685,13 @@ class Network(Plottable):
           in the graph (after sorting, to evade cycles).
 
         """
-        ## Sort by execution order, then by operation-insertion, to break ties.
-        ordered_nodes = iset(self._topo_sort_nodes(pruned_dag))
-
+        ## Sort by execution order, then by operation-insertion, to break ties,
+        #  and inform user in case of cycles (must have been caught earlier) .
+        #
         if not outputs or is_skip_evictions():
             # When no specific outputs asked, NO EVICTIONS,
             # so just add the Operations.
-            return list(yield_ops(ordered_nodes))
+            return list(op for op in yield_ops(sorted_nodes) if op in pruned_dag)
 
         ## Strip SFXED without the fear of cycles
         #  (must augment dag before stripping outputs docchains).
@@ -675,13 +720,13 @@ class Network(Plottable):
             steps.append(dep)
 
         steps = []
-        for i, op in enumerate(ordered_nodes):
-            if not isinstance(op, Operation):
+        for i, op in enumerate(sorted_nodes):
+            if not isinstance(op, Operation) or op not in pruned_dag:
                 continue
 
             steps.append(op)
 
-            future_nodes = set(ordered_nodes[i + 1 :])
+            future_nodes = sorted_nodes[i + 1 :]
 
             ## EVICT(1) operation's needs not to be used in the future.
             #
@@ -756,58 +801,76 @@ class Network(Plottable):
             the cached or fresh new :term:`execution plan`
 
         :raises ValueError:
-            - If `outputs` asked do not exist in network, with msg:
-
-                *Unknown output nodes: ...*
-
-            - If solution does not contain any operations, with msg:
-
-                *Unsolvable graph: ...*
-
-            - If given `inputs` mismatched plan's :attr:`needs`, with msg:
-
-                *Plan needs more inputs...*
-
-            - If net cannot produce asked `outputs`, with msg:
-
-                *Unreachable outputs...*
+            *Unknown output nodes...*
+                if `outputs` asked do not exist in network.
+            *Unsolvable graph: ...*
+                if it cannot produce any `outputs` from the given `inputs`.
+            *Plan needs more inputs...*
+                if given `inputs` mismatched plan's :attr:`needs`.
+            *Unreachable outputs...*
+                if net cannot produce asked `outputs`.
         """
         from .execution import ExecutionPlan
 
-        ## Make a stable cache-key.
-        #
-        if inputs is not None:
-            inputs = tuple(
-                sorted(astuple(inputs, "inputs", allowed_types=abc.Collection))
-            )
-        if outputs is not None:
-            outputs = tuple(
-                sorted(astuple(outputs, "outputs", allowed_types=abc.Collection))
-            )
-        if not predicate:
-            predicate = None
+        ok = False
+        try:
+            ## Make a stable cache-key.
+            #
+            if inputs is not None:
+                inputs = tuple(
+                    sorted(astuple(inputs, "inputs", allowed_types=abc.Collection))
+                )
+            if outputs is not None:
+                outputs = tuple(
+                    sorted(astuple(outputs, "outputs", allowed_types=abc.Collection))
+                )
+            if not predicate:
+                predicate = None
 
-        cache_key = (inputs, outputs, predicate)
+            cache_key = (inputs, outputs, predicate)
 
-        ## Build (or retrieve from cache) execution plan
-        #  for the given inputs & outputs.
-        #
-        if cache_key in self._cached_plans:
-            log.debug("... compile cache-hit key: %s", cache_key)
-            plan = self._cached_plans[cache_key]
-        else:
-            pruned_dag, needs, provides = self._prune_graph(inputs, outputs, predicate)
-            steps = self._build_execution_steps(pruned_dag, needs, outputs or ())
-            plan = ExecutionPlan(
-                self,
-                needs,
-                provides,
-                pruned_dag,
-                tuple(steps),
-                asked_outs=outputs is not None,
-            )
+            ## Build (or retrieve from cache) execution plan
+            #  for the given inputs & outputs.
+            #
+            if cache_key in self._cached_plans:
+                log.debug("... compile cache-hit key: %s", cache_key)
+                plan = self._cached_plans[cache_key]
+            else:
+                (
+                    pruned_dag,
+                    sorted_nodes,
+                    needs,
+                    provides,
+                    op_comments,
+                ) = self._prune_graph(inputs, outputs, predicate)
+                steps = self._build_execution_steps(
+                    pruned_dag, sorted_nodes, needs, outputs or ()
+                )
+                plan = ExecutionPlan(
+                    self,
+                    needs,
+                    provides,
+                    pruned_dag,
+                    tuple(steps),
+                    asked_outs=outputs is not None,
+                    comments=op_comments,
+                )
 
-            self._cached_plans[cache_key] = plan
-            log.debug("... compile cache-updated key: %s", cache_key)
+                self._cached_plans[cache_key] = plan
+                log.debug("... compile cache-updated key: %s", cache_key)
 
-        return plan
+            ok = True
+            return plan
+        finally:
+            if not ok:
+                ex = sys.exc_info()[1]
+                save_jetsam(
+                    ex,
+                    locals(),
+                    "pruned_dag",
+                    "sorted_nodes",
+                    "needs",
+                    "provides",
+                    "op_comments",
+                    "plan",
+                )
