@@ -23,7 +23,8 @@ from collections import abc, namedtuple
 from contextlib import contextmanager
 from contextvars import ContextVar
 from functools import partial
-from itertools import chain
+from itertools import chain, count
+from pathlib import Path
 from typing import (
     Any,
     Callable,
@@ -37,13 +38,13 @@ from typing import (
 )
 
 import jinja2
-from jinja2.filters import do_truncate
 import networkx as nx
 import pydot
 from boltons.iterutils import default_enter, default_exit, get_path, remap
+from jinja2.filters import do_truncate
 
 from . import __version__
-from .base import Operation, PlotArgs, first_solid, func_name, func_source
+from .base import Operation, PlotArgs, Plottable, first_solid, func_name, func_source
 from .config import (
     is_debug,
     is_endure_operations,
@@ -51,7 +52,8 @@ from .config import (
     is_parallel_tasks,
     is_reschedule_operations,
 )
-from .modifier import is_sfx, is_sfxed
+from .modifier import get_jsonp, is_sfx, is_sfxed
+from .planning import yield_node_names
 
 log = logging.getLogger(__name__)
 
@@ -68,7 +70,7 @@ log = logging.getLogger(__name__)
 #:     Jupyter notebooks, as defined in https://github.com/ariutta/svg-pan-zoom#how-to-use
 #:     if `None`, defaults to string (also maps supported)::
 #:
-#:             "{controlIconsEnabled: true, zoomScaleSensitivity: 0.4, fit: true}"
+#:             "{controlIconsEnabled: true, fit: true}"
 #:
 #: :param svg_element_styles:
 #:     mostly for sizing the zoomable SVG in Jupyter notebooks.
@@ -79,6 +81,10 @@ log = logging.getLogger(__name__)
 #:
 #: :param svg_container_styles:
 #:     like `svg_element_styles`, if `None`, defaults to empty string (also maps supported).
+#:
+#: .. note::
+#:      referred also by :rst:dir:`graphtik`'s  :confval:`graphtik_zoomable_options`
+#:      default configuration value.
 default_jupyter_render = {
     "svg_pan_zoom_json": "{controlIconsEnabled: true, fit: true}",
     "svg_element_styles": "width: 100%; height: 300px;",
@@ -334,6 +340,9 @@ def _make_jinja2_environment() -> jinja2.Environment:
     env.filters["hrefer"] = _drop_gt_lt
     env.filters["ex"] = _format_exception
     env.filters["truncate"] = _reversing_truncate
+    env.filters["sideffected"] = lambda x: is_sfx(x) or None
+    env.filters["sfxed"] = env.filters["sfx_list"] = lambda x: is_sfxed(x) or None
+    env.filters["jsonp"] = get_jsonp
 
     return env
 
@@ -355,6 +364,16 @@ def make_template(s):
     ``hrefer``
         Dubious escape for when writting URLs inside Graphviz attributes.
         Does NOT collapse nones/empties (like default ``e``)
+    ``ex``
+        format exceptions
+    ``truncate``
+        reversing truncate (keep tail) if `truncate` arg is true
+    ``sideffected``
+        return the `sideffected` part of an `sfxed` or none
+    ``sfx_list``
+        return the `sfx_list` part of an `sfxed` or none
+    ``jsonp``
+        return the `jsonp` list of a dependency or none
     """
     return _jinja2_env.from_string(textwrap.dedent(s).strip())
 
@@ -373,6 +392,21 @@ def make_data_value_tooltip(plot_args: PlotArgs):
         return quote_html_tooltips(tooltip)
 
 
+def make_op_prune_comment(plot_args: PlotArgs):
+    op = plot_args.nx_item
+    sol = plot_args.solution
+    plottable = plot_args.plottable
+
+    comments = ()
+    if sol is not None and op in sol.plan.comments:
+        comments = sol.plan.comments
+    elif op in getattr(plottable, "comments", ()):
+        comments = plottable.comments
+
+    if comments:
+        return f"(pruned due to {comments[op]})"
+
+
 def make_op_tooltip(plot_args: PlotArgs):
     """the string-representation of an operation (name, needs, provides)"""
     return plot_args.nx_attrs.get("_op_tooltip", str(plot_args.nx_item))
@@ -390,7 +424,8 @@ class Theme:
     The poor man's css-like :term:`plot theme` (see also :class:`.StyleStack`).
 
     To use the values contained in theme-instances, stack them in a :class:`.StylesStack`,
-    and :meth:`.StylesStack.merge` them with :term:`style expansion`\\s.
+    and :meth:`.StylesStack.merge` them with :term:`style expansion`\\s
+    (read it from :meth:`.StyleStack.expand()`).
 
     .. theme-warn-start
     .. Attention::
@@ -414,6 +449,7 @@ class Theme:
     #: args for jinja2 patched `truncate` filter, above.
     truncate_args = ((23, True), {"reverse": True})
     fill_color = "wheat"
+    sideffect_color = "blue"
     subdoc_color = "#8B4513"  # SaddleBrown
     pruned_color = "#d3d3d3"  # LightGrey
     canceled_color = "#a9a9a9"  # DarkGray
@@ -422,7 +458,7 @@ class Theme:
     broken_color = "Red"
     overwrite_color = "SkyBlue"
     steps_color = "#00bbbb"
-    evicted = "#006666"
+    evicted_color = "#006666"
     #: the url to the architecture section explaining *graphtik* glossary,
     #: linked by legend.
     arch_url = "https://graphtik.readthedocs.io/en/latest/arch.html"
@@ -451,39 +487,47 @@ class Theme:
     #: in :attr:`.kw_plottable_type` ( ot missing altogether).
     kw_graph_plottable_type_unknown = {}
 
+    #: Attributes applying to all nodes with ``node [...]`` graphviz construct,
+    #: append in graph only if non-empty.
+    node_defaults = {}
+    #: Attributes applying to all edges with ``edge [...]`` graphviz construct,
+    #: appended in graph only if non-empty.
+    edge_defaults = {}
+
     ##########
     ## DATA node
     ##
 
+    #: Keys to ignore from data styles & node-attrs,
+    #: because they are handled internally by HTML-Label, and/or
+    #: interact badly with that label.
+    data_bad_html_label_keys = {"label"}
+
+    #: Jinja2 params for the HTML-Table label
+    kw_data_label = {}
     #: Reduce margins, since sideffects take a lot of space
     #: (default margin: x=0.11, y=0.055O)
     kw_data = {
         "shape": "rect",
-        "margin": "0.04,0.02",
-        "label": make_template("{{ nx_item | truncate }}"),
+        "fixedsize": "shape",
     }
     kw_data_inp = {}
     kw_data_out = {}
-    kw_data_inp_only = {"shape": "invhouse"}
-    kw_data_out_only = {"shape": "house"}
-    kw_data_io = {"shape": "hexagon"}
+    kw_data_inp_only = {"shape": "invhouse", "tooltip": ["(input)"]}
+    kw_data_out_only = {"shape": "house", "tooltip": ["(output)"]}
+    kw_data_io = {
+        "shape": "hexagon",
+        "tooltip": ["(input+output)"],
+    }
     kw_data_sideffect = {
-        "color": "blue",
-        "fontcolor": "blue",
-    }
-    kw_data_sideffected = {
-        "label": make_template(
-            """
-            <{{ nx_item.sideffected | eee }}<BR/>
-            (<I>sfx:</I> {{ nx_item.sfx_list | join(', ') | eee }})>
-            """
-        ),
+        "color": Ref("sideffect_color"),
+        "fontcolor": Ref("sideffect_color"),
     }
+    kw_data_sideffected = {}
     kw_data_to_evict = {
-        "color": Ref("evicted"),
-        "fontcolor": Ref("evicted"),
+        "color": Ref("evicted_color"),
         "style": ["dashed"],
-        "tooltip": "(to evict)",
+        "tooltip": ["(to evict)"],
     }
     ##
     ## data STATE
@@ -491,20 +535,71 @@ class Theme:
     kw_data_pruned = {
         "fontcolor": Ref("pruned_color"),
         "color": Ref("pruned_color"),
-        "tooltip": "(pruned)",
+        "tooltip": ["(pruned)"],
     }
     kw_data_in_solution = {
         "style": ["filled"],
         "fillcolor": Ref("fill_color"),
-        "tooltip": make_data_value_tooltip,
+        "tooltip": [make_data_value_tooltip],
     }
-    kw_data_evicted = {"penwidth": "3", "tooltip": "(evicted)"}
-    kw_data_overwritten = {"style": ["filled"], "fillcolor": Ref("overwrite_color")}
-    kw_data_canceled = {
-        "fillcolor": Ref("canceled_color"),
+    kw_data_evicted = {"penwidth": "3", "tooltip": ["(evicted)"]}
+    kw_data_overwritten = {
         "style": ["filled"],
-        "tooltip": "(canceled)",
+        "fillcolor": Ref("overwrite_color"),
+        "tooltip": ["(overridden)"],
     }
+    kw_data_missing = {
+        "fontcolor": Ref("canceled_color"),
+        "color": Ref("canceled_color"),
+        "tooltip": ["(missing-optional or canceled)"],
+    }
+    data_template = make_template(
+        """\
+        <<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+            <TR><TD>
+                {%- if nx_item | jsonp -%}
+                    {%- for js_step in nx_item | jsonp -%}
+                        {%- if loop.first -%}
+                            {{- js_step | truncate -}}
+                            /
+                        {%- else -%}
+                            {{- '\n' -}}
+                            {{- '  ' * (loop.index - 1) -}}
+                            +--
+                            {{- js_step | truncate -}}
+                            {%- if not loop.last -%}
+                                /
+                            {%- endif -%}
+                        {%- endif -%}
+                        <BR ALIGN="LEFT"/>
+                    {%- endfor -%}
+                {%- elif nx_item | sfxed -%}
+                    {{- nx_item | sideffected | truncate -}}
+                {%- else -%}
+                    {{- nx_item | truncate -}}
+                {%- endif -%}
+
+                {%- if nx_item | sfxed -%}
+                    <BR/>(<I>sfx:</I> {{ nx_item | sfx_list | join(', ') | eee }})
+                {%- endif -%}
+
+            </TD>
+            {%- if steps and nx_item in steps %}
+            <TD STYLE="rounded" CELLSPACING="2" CELLPADDING="4"
+            {{- {
+                'BGCOLOR': step_bgcolor | eee,
+                'TITLE': step_tooltip | eee,
+                'HREF': step_url | hrefer | ee,
+                'TARGET': step_target | eee
+                } | xmlattr }}
+            ><FONT FACE="monospace" COLOR="{{ step_color | eee }}"><B>
+                    {{- steps.index(nx_item) -}}
+                </B></FONT></TD>
+            {%- endif -%}
+            </TR>
+        </TABLE>>
+        """
+    )
 
     ##########
     ## OPERATION node
@@ -519,20 +614,20 @@ class Theme:
         "name": lambda pa: quote_node_id(pa.nx_item.name),
         "shape": "plain",  # dictated by Graphviz docs
         # Set some base tooltip, or else, "TABLE" shown...
-        "tooltip": lambda pa: graphviz_html_string(pa.nx_item.name),
+        "tooltip": [lambda pa: graphviz_html_string(pa.nx_item.name)],
     }
 
     kw_op_executed = {"fillcolor": Ref("fill_color")}
     kw_op_endured = {
         "penwidth": Ref("resched_thickness"),
         "style": ["dashed"],
-        "tooltip": "(endured)",
+        "tooltip": ["(endured)"],
         "badges": ["!"],
     }
     kw_op_rescheduled = {
         "penwidth": Ref("resched_thickness"),
         "style": ["dashed"],
-        "tooltip": "(rescheduled)",
+        "tooltip": ["(rescheduled)"],
         "badges": ["?"],
     }
     kw_op_parallel = {"badges": ["|"]}
@@ -542,11 +637,12 @@ class Theme:
     ## op STATE
     ##
     kw_op_pruned = {"color": Ref("pruned_color"), "fontcolor": Ref("pruned_color")}
+    kw_op_prune_comment = {"op_tooltip": [make_op_prune_comment]}
     kw_op_failed = {
         "fillcolor": Ref("failed_color"),
-        "tooltip": make_template("{{ solution.executed[nx_item] if solution | ex }}"),
+        "tooltip": [make_template("{{ solution.executed[nx_item] if solution | ex }}")],
     }
-    kw_op_canceled = {"fillcolor": Ref("canceled_color"), "tooltip": "(canceled)"}
+    kw_op_canceled = {"fillcolor": Ref("canceled_color"), "tooltip": ["(canceled)"]}
     #: Operation styles may specify one or more "letters"
     #: in a `badges` list item, as long as the "letter" is contained in the dictionary
     #: below.
@@ -557,52 +653,60 @@ class Theme:
                 "bgcolor": "#04277d",
                 "color": "white",
                 "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-endured",
+                "target": "_top",
             },
             "?": {
                 "tooltip": "rescheduled",
                 "bgcolor": "#fc89ac",
                 "color": "white",
                 "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-partial-outputs",
+                "target": "_top",
             },
             "|": {
                 "tooltip": "parallel",
                 "bgcolor": "#b1ce9a",
                 "color": "white",
                 "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-parallel-execution",
+                "target": "_top",
             },
             "&": {
                 "tooltip": "marshalled",
                 "bgcolor": "#4e3165",
                 "color": "white",
                 "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-marshalling",
+                "target": "_top",
             },
             "}": {
                 "tooltip": "returns_dict",
                 "bgcolor": "#cc5500",
                 "color": "white",
                 "URL": "https://graphtik.readthedocs.io/en/latest/arch.html#term-returns-dictionary",
+                "target": "_top",
             },
         }
     }
-    #: props of the HTML-Table label for Operations
+    #: Jinja2 params for the HTML-Table label, applied 1ST.
     kw_op_label = {
         "op_name": lambda pa: pa.nx_item.name,
         "fn_name": lambda pa: pa.nx_item
         and func_name(pa.nx_item.fn, mod=1, fqdn=1, human=1),
         "op_truncate": Ref("truncate_args"),
         "fn_truncate": Ref("truncate_args"),
-        "op_tooltip": make_op_tooltip,
-        "fn_tooltip": make_fn_tooltip,
         "op_url": Ref("op_url", default=None),
         "op_link_target": "_top",
         "fn_url": Ref("fn_url", default=None),
         "fn_link_target": "_top",
     }
+    #: Jinja2 params for the HTML-Table label applied AT THE END.
+    kw_op_label2 = {
+        "op_tooltip": [make_op_tooltip],  # ensure
+        "fn_tooltip": [make_fn_tooltip],
+    }
     #: Try to mimic a regular `Graphviz`_ node attributes
     #: (see examples in ``test.test_plot.test_op_template_full()`` for params).
     #: TODO: fix jinja2 template is un-picklable!
     op_template = make_template(
-        """
+        """\
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded"
           {{- {
           'BORDER': penwidth | ee,
@@ -614,26 +718,38 @@ class Theme:
                   {{- {
                   'TOOLTIP': op_tooltip | truncate | eee,
                   'HREF': op_url | hrefer | ee,
-                  'TARGET': op_link_target | e
+                  'TARGET': op_link_target | eee
                   } | xmlattr }}
                 >
                     {%- if fontcolor -%}<FONT COLOR="{{ fontcolor }}">{%- endif -%}
                     {{- '<B>OP:</B> <I>%s</I>' % op_name | truncate(*op_truncate[0], **op_truncate[1]) | ee if op_name -}}
                     {%- if fontcolor -%}</FONT>{%- endif -%}
                 </TD>
-                <TD BORDER="1" SIDES="b">
-                {%- if badges -%}
-                    <TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
+                <TD BORDER="1" SIDES="b" ALIGN="right">
+                {%- if badges or (steps and op_name in steps) -%}
+                    <TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2" ALIGN="right">
                         <TR>
-                        {%- for badge in badges -%}
-                            <TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="{{ badge_styles[badge].bgcolor
-                                }}" TITLE="{{ badge_styles[badge].tooltip | e
+                        {%- if steps and op_name in steps %}
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="14" FIXEDSIZE="true" VALIGN="BOTTOM"
+                            {{- {
+                            'BGCOLOR': step_bgcolor | eee,
+                            'TITLE': step_tooltip | eee,
+                            'HREF': step_url | hrefer | ee,
+                            'TARGET': step_target | eee
+                            } | xmlattr -}}
+                            ><FONT FACE="monospace" COLOR="{{ step_color | eee }}"><B>
+                                {{- steps.index(op_name) | eee -}}
+                            </B></FONT></TD>
+                        {%- endif -%}
+                        {%- for badge in badges %}
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="{{ badge_styles[badge].bgcolor
+                                }}" TITLE="{{ badge_styles[badge].tooltip | eee
                                 }}" HREF="{{ badge_styles[badge].URL | hrefer | ee
-                                }}" TARGET="_blank"
+                                }}" TARGET="{{ badge_styles[badge].target }}"
                             ><FONT FACE="monospace" COLOR="{{ badge_styles[badge].color }}"><B>
                                 {{- badge | eee -}}
                             </B></FONT></TD>
-                        {%- endfor -%}
+                        {%- endfor %}
                         </TR>
                     </TABLE>
                 {%- endif -%}
@@ -645,7 +761,7 @@ class Theme:
                   {{- {
                   'TOOLTIP': fn_tooltip | truncate | eee,
                   'HREF': fn_url | hrefer | ee,
-                  'TARGET': fn_link_target | e
+                  'TARGET': fn_link_target | eee
                   } | xmlattr }}
                   >
                     {%- if fontcolor -%}
@@ -669,8 +785,16 @@ class Theme:
     kw_edge = {"tailport": "s", "headport": "n"}
     kw_edge_tail_op = {}
     kw_edge_head_op = {"arrowtail": "inv", "dir": "back"}
-    kw_edge_optional = {"style": ["dashed"]}
-    kw_edge_sideffect = {"color": "blue"}
+    kw_edge_optional = {"style": ["dashed",], "tooltip": ["(optional)"]}
+    kw_edge_sideffect = {"color": Ref("sideffect_color")}
+    kw_edge_implicit = {
+        lambda pa: (
+            "arrowtail" if isinstance(pa.nx_item[0], Operation) else "arrowhead"
+        ): "obox",
+        "dir": "both",
+        "tooltip": ["(implicit)"],
+        "fontcolor": Ref("sideffect_color"),
+    }
     #: Added conditionally if `alias_of` found in edge-attrs.
     kw_edge_alias = {
         "fontsize": 11,  # default: 14
@@ -682,7 +806,10 @@ class Theme:
     kw_edge_mapping_keyword = {
         "fontsize": 11,  # default: 14
         "fontname": "italic",
-        "label": make_template("<&gt;{{ nx_attrs['keyword'] | eee }}>"),
+        "tooltip": ["(mapped-fn-keyword)"],
+        "label": make_template(
+            "**{{ nx_attrs['keyword'] | eee if nx_attrs['keyword'] != nx_item[0] }}"
+        ),
     }
     kw_edge_subdoc = {
         "color": Ref("subdoc_color"),
@@ -690,23 +817,44 @@ class Theme:
         "dir": "back",
         "tailport": "se",
         "headport": "nw",
+        "tooltip": ["(subdoc)"],
     }
 
-    kw_edge_pruned = {"color": Ref("pruned_color")}
+    kw_edge_pruned = {"color": Ref("pruned_color"), "fontcolor": Ref("pruned_color")}
     kw_edge_rescheduled = {"style": ["dashed"]}
     kw_edge_endured = {"style": ["dashed"]}
-    kw_edge_broken = {"color": Ref("broken_color")}
+    kw_edge_broken = {"tooltip": ["(partial-broken)"], "color": Ref("broken_color")}
 
     ##########
     ## Other
     ##
 
-    #: When true, plot also :term:`execution steps`, linking operations and evictions
-    #: with green dotted lines labeled with numbers denoting the execution order.
-    show_steps = False
-    #: When true, plot also :term:`hierarchical data` nodes that
-    #: are not directly linked to operations.
-    show_chaindocs = False
+    #: `None`:
+    #:      plot just a badge with the order (a number) of each op/data
+    #:      in steps (if contained);
+    #: true:
+    #:      plot also :term:`execution steps`, linking operations and evictions
+    #:      with green dotted lines labeled with numbers denoting the execution order;
+    #: false:
+    #:      hide even op/data step order badges.
+    show_steps = None
+    #: `None`:
+    #:  hide any parent/subdoc not related directly to some operation;
+    #: true:
+    #:  plot also :term:`hierarchical data` nodes not directly linked
+    #:  to operations;
+    #: false:
+    #:  hide also parent-subdoc relation edges.
+    show_chaindocs = None
+    #: Available as jinja2 params for both data & operation templates.
+    kw_step_badge = {
+        "step_bgcolor": Ref("steps_color"),
+        "step_color": "white",
+        "step_url": "https://graphtik.readthedocs.io/en/latest/arch.html#term-steps",
+        "step_target": "_top",
+        "step_tooltip": "computation order",
+    }
+    #: step edges
     kw_step = {
         "style": "dotted",  # Note: Step styles are not *remerged*.`
         "color": Ref("steps_color"),
@@ -938,7 +1086,7 @@ class StylesStack(NamedTuple):
         """
         if nx_attrs:
             kw = {
-                k.lstrip(USER_STYLE_PREFFIX): v
+                k[len(USER_STYLE_PREFFIX) :]: v
                 for k, v in nx_attrs.items()
                 if k.startswith(USER_STYLE_PREFFIX) and k not in skip
             }
@@ -951,20 +1099,22 @@ class StylesStack(NamedTuple):
 
         .. theme-expansions-start
 
+        - Call any *callables* found as keys, values or the whole style-dict,
+          passing in the current :class:`plot_args <.PlotArgs>`, and replace those
+          with the callable's result (even more flexible than templates).
+
         - Resolve any :class:`.Ref` instances, first against the current *nx_attrs*
           and then against the attributes of the current theme.
 
-        - Render jinja2 templates (see :meth:`_expand_styles()`) with template-arguments
-          all the attributes of the :class:`plot_args <.PlotArgs>` instance in use
-          (hence much more flexible than :class:`.Ref`).
-
-        - Call any *callables* with current :class:`plot_args <.PlotArgs>` and
-          replace them by their result (even more flexible than templates).
+        - Render jinja2 templates with template-arguments all attributes of :class:`plot_args
+          <.PlotArgs>` instance in use, (hence much more flexible than :class:`.Ref`).
 
         - Any Nones results above are discarded.
 
         - Workaround pydot/pydot#228 pydot-cstor not supporting styles-as-lists.
 
+        - Merge tooltip-lists.
+
         .. theme-expansions-end
         """
 
@@ -974,15 +1124,28 @@ class StylesStack(NamedTuple):
             """
             visit_type = type(v).__name__
             try:
+                expanded = False
+                if callable(k):
+                    k = k(self.plot_args)
+                    expanded = True
+
                 if isinstance(v, Ref):
                     v = v.resolve(self.plot_args.nx_attrs, self.plot_args.theme)
+                    expanded = True
                 elif isinstance(v, jinja2.Template):
                     v = v.render(**self.plot_args._asdict())
+                    expanded = True
                 elif callable(v):
                     v = v(self.plot_args)
-                else:
-                    return False if v in (..., None) else True
-                return False if v in (None, ...) else (k, v)
+                    expanded = True
+
+                return (
+                    False
+                    if k is None or v in (..., None)
+                    else (k, v)
+                    if expanded
+                    else True
+                )
             except Exception as ex:
                 path = f'{"/".join(path)}/{k}'
                 msg = f"Failed expanding {visit_type} @ '{path}' = {v!r} due to: {type(ex).__name__}({ex})"
@@ -994,6 +1157,8 @@ class StylesStack(NamedTuple):
 
         style = remap(style, visit=expand_visitor)
 
+        ## Merge Graphviz-style props
+        #
         graphviz_style = style.get("style")
         if isinstance(graphviz_style, (list, tuple)):
             ## FIXME: support only plain-strings as graphviz-styles.
@@ -1002,6 +1167,14 @@ class StylesStack(NamedTuple):
                 graphviz_style = f'"{graphviz_style}"'
             style["style"] = graphviz_style
 
+        ## Merge tooltips
+        #
+        for k in style.keys():
+            if k.endswith("tooltip"):
+                tooltips = style[k]
+                if isinstance(tooltips, (list, tuple)):
+                    style[k] = "\n".join(tooltips)
+
         return style
 
     def merge(self, debug=None) -> dict:
@@ -1009,14 +1182,13 @@ class StylesStack(NamedTuple):
         Recursively merge :attr:`named_styles` and :meth:`.expand` the result style.
 
         :param debug:
-            When not `None`, override :func:`config.is_debug` flag.
-            When debug is enabled, tooltips are overridden with provenance
-            & nx_attrs.
+            When not `None`, override :ref:`debug`;  when enabled,
+            tooltips are overridden with provenance & nx_attrs.
 
         :return:
             the merged styles
         """
-
+        # FIXME: too clever, avoided re-writting own merge, and now errors cannot debug :-(
         if not self.named_styles:
             return {}
 
@@ -1030,13 +1202,28 @@ class StylesStack(NamedTuple):
             ## Append debug info
             #
             provenance_str = pformat(
-                {".".join(k): v for k, v in styles_provenance.items()}, indent=2
+                {
+                    ".".join(
+                        # resolve any callable theme-keys
+                        str(k_step(self.plot_args)) if callable(k_step) else k_step
+                        for k_step in k
+                    ): v
+                    for k, v in styles_provenance.items()
+                },
+                indent=2,
             )
             tooltip = f"- styles: {provenance_str}\n- extra_attrs: {pformat(self.plot_args.nx_attrs)}"
             style["tooltip"] = graphviz_html_string(tooltip)
 
         else:
             style = remerge(*(style_dict for _name, style_dict in self.named_styles))
+            style = remerge(
+                *(
+                    ## Expand also callables producing the whole style-dict
+                    style_dict(self.plot_args) if callable(style_dict) else style_dict
+                    for _name, style_dict in self.named_styles
+                )
+            )
         assert isinstance(style, dict), (style, self.named_styles)
 
         return self.expand(style)
@@ -1102,14 +1289,26 @@ class Plotter:
 
         styles.stack_user_style(graph.graph)
 
-        kw = styles.merge()
-        dot = pydot.Dot(**kw)
-        ## Item-args for nodes, edges & steps spring off of this.
-        base_plot_args = plot_args._replace(dot=dot, clustered={})
+        dot = pydot.Dot(**styles.merge())
 
         if plot_args.name:
             dot.set_name(as_identifier(plot_args.name))
 
+        ## Item-args for nodes, edges & steps spring off of this.
+        plot_args = plot_args._replace(dot=dot, clustered={})
+
+        ## Graph's node/edge defaults
+        #
+        if theme.node_defaults:
+            node_styles = self._new_styles_stack(plot_args)
+            node_styles.add("node_defaults")
+            dot.add_node(pydot.Node("node", **node_styles.merge()))
+        if theme.edge_defaults:
+            edge_styles = self._new_styles_stack(plot_args)
+            edge_styles.add("edge_defaults")
+            dot.add_node(pydot.Node("edge", **edge_styles.merge()))
+
+        # To hide edges from unused chain-docs.
         hidden = set()
 
         ## NODES
@@ -1119,20 +1318,28 @@ class Plotter:
                 hidden.add(nx_node)
                 continue
 
-            plot_args = base_plot_args._replace(nx_item=nx_node, nx_attrs=data)
-            dot_node = self._make_node(plot_args)
-            plot_args = plot_args._replace(dot_item=dot_node)
+            node_plot_args = plot_args._replace(nx_item=nx_node, nx_attrs=data)
+            dot_node = self._make_node(node_plot_args)
+            node_plot_args = node_plot_args._replace(dot_item=dot_node)
 
-            self._append_or_cluster_node(plot_args)
+            self._append_or_cluster_node(node_plot_args)
 
         ## EDGES
         #
         for src, dst, data in graph.edges.data(True):
-            if src in hidden or dst in hidden:
+            if (
+                src in hidden
+                or dst in hidden
+                or (
+                    data.get("subdoc")
+                    and not theme.show_chaindocs
+                    and theme.show_chaindocs is not None
+                )
+            ):
                 continue
 
-            plot_args = base_plot_args._replace(nx_item=(src, dst), nx_attrs=data)
-            dot.add_edge(self._make_edge(plot_args))
+            edge_plot_args = plot_args._replace(nx_item=(src, dst), nx_attrs=data)
+            dot.add_edge(self._make_edge(edge_plot_args))
 
         ## Draw steps sequence, if it's worth it.
         #
@@ -1143,7 +1350,7 @@ class Plotter:
             for i, (src, dst) in enumerate(zip(it1, it2), 1):
                 src_name = get_node_name(src)
                 dst_name = get_node_name(dst)
-                styles = self._new_styles_stack(base_plot_args)
+                styles = self._new_styles_stack(plot_args)
 
                 styles.add("kw_step")
                 edge = pydot.Edge(
@@ -1186,10 +1393,13 @@ class Plotter:
         nx_node = plot_args.nx_item
         node_attrs = plot_args.nx_attrs
         (plottable, _, _, steps, inputs, outputs, solution, *_,) = plot_args
-        if solution is not None and steps is None:
-            steps = solution.plan.steps
 
+        label_styles = self._new_styles_stack(plot_args)
         if isinstance(nx_node, str):  # DATA
+            label_styles.add("kw_data_label")
+            label_styles.stack_user_style(node_attrs)
+            label_styles.add("kw_step_badge")
+
             styles = self._new_styles_stack(plot_args)
 
             styles.add("kw_data")
@@ -1242,13 +1452,22 @@ class Plotter:
                 elif nx_node in steps:
                     styles.add("kw_data_evicted")
                 elif not is_pruned and not is_sfx(nx_node):
-                    styles.add("kw_data_canceled")
+                    styles.add("kw_data_missing")
 
-            styles.stack_user_style(node_attrs)
+            styles.stack_user_style(node_attrs, skip=theme.data_bad_html_label_keys)
+
+            styles.add(
+                "data_label_template",
+                {
+                    "label": _render_template(
+                        theme.data_template,
+                        **plot_args._asdict(),
+                        **label_styles.merge(),
+                    )
+                },
+            )
 
         else:  # OPERATION
-            label_styles = self._new_styles_stack(plot_args)
-
             label_styles.add("kw_op_label")
 
             ## Op-kind
@@ -1266,8 +1485,11 @@ class Plotter:
 
             ## Op-state
             #
+
             if steps and nx_node not in steps:
                 label_styles.add("kw_op_pruned")
+            label_styles.add("kw_op_prune_comment")
+
             if solution:
                 if solution.is_failed(nx_node):
                     label_styles.add("kw_op_failed")
@@ -1280,20 +1502,32 @@ class Plotter:
 
             # TODO: Optimize and merge badge_styles once!
             label_styles.add("op_badge_styles")
+            label_styles.add("kw_step_badge")
+            label_styles.add("kw_op_label2")
 
-            kw = label_styles.merge()
             styles = self._new_styles_stack(plot_args)
             styles.add("kw_op")
+
+            # Decide if steps badge would be shown.
+            #
+            if not steps or not theme.show_steps and theme.show_steps is not None:
+                steps = None
+            else:
+                steps = steps and list(yield_node_names(steps))
+
             styles.add(
                 "op_label_template",
-                {"label": _render_template(theme.op_template, **kw,)},
+                {
+                    "label": _render_template(
+                        theme.op_template, steps=steps, **label_styles.merge(),
+                    )
+                },
             )
 
             # Exclude Graphviz node attributes interacting badly with HTML-Labels.
             styles.stack_user_style(node_attrs, skip=theme.op_bad_html_label_keys)
 
-        kw = styles.merge()
-        return pydot.Node(**kw)
+        return pydot.Node(**styles.merge())
 
     def _build_cluster_path(self, plot_args: PlotArgs, *path,) -> None:
         """Return the last cluster in the path created"""
@@ -1356,6 +1590,8 @@ class Plotter:
             styles.add("kw_edge_optional")
         if edge_attrs.get("sideffect"):
             styles.add("kw_edge_sideffect")
+        if edge_attrs.get("implicit"):
+            styles.add("kw_edge_implicit")
         if edge_attrs.get("alias_of"):
             styles.add("kw_edge_alias")
         if edge_attrs.get("keyword"):
@@ -1380,8 +1616,7 @@ class Plotter:
             styles.add("kw_edge_broken")
 
         styles.stack_user_style(edge_attrs)
-        kw = styles.merge()
-        edge = pydot.Edge(src=src_name, dst=dst_name, **kw)
+        edge = pydot.Edge(src=src_name, dst=dst_name, **styles.merge())
 
         return edge
 
@@ -1534,14 +1769,16 @@ class Plotter:
         ## From https://stackoverflow.com/questions/3499056/making-a-legend-key-in-graphviz
         # Render it manually with these python commands, and remember to update result in git:
         #
-        #   from graphtik.plot import legend
-        #   legend('docs/source/images/GraphtikLegend.svg')
+        #   from graphtik import plot
+        #   plot.legend('docs/source/images/GraphtikLegend.svg')
         dot_text = (
             """
         digraph {
             rankdir=TB;
             subgraph cluster_legend {
             label="Graphtik Legend %(ver)s";
+            edge [target=_top];
+            node [target=_top];
 
             Dependencies   [shape=plaintext fontsize=16 fontname="bold italic"];
             Dependencies -> needs [dir=back arrowtail=inv
@@ -1550,11 +1787,15 @@ class Plotter:
             needs [color=invis];
             needs -> provides     [tooltip="Compulsory dependency from src Operation --> dst `provides` data"];
             provides [color=invis];
-            provides -> optional     [style=dashed];
+            provides -> optional     [style=dashed label="**kw"];
             optional    [color=invis label="optional\npartial out"
                         tooltip="Target operation may run without source `need` OR source operation may not `provide` target data."
                         URL="%(arch_url)s#term-needs"];
-            optional    -> sideffect    [color=blue];
+            optional    -> implicit [dir=both arrowtail=inv arrowhead=obox];
+            implicit    [color=invis label="implicit"
+                        tooltip="A `need` passed in as **kw argument underlying fn."
+                        URL="%(arch_url)s#term-implicit"];
+            implicit    -> sideffect    [color=blue];
             sideffect   [color=invis
                         tooltip="Fictive data not consumed/produced by operation functions."
                         URL="%(arch_url)s#term-sideffects"];
@@ -1576,24 +1817,26 @@ class Plotter:
 
             DataLabel   [shape=plaintext fontsize=16 fontname="bold italic"
                             label=Data];
-            data    [shape=rect
+            data    [shape=rect fixedsize=shape
                     tooltip="Any data not given or asked."
                     URL="%(arch_url)s#term-graph"];
-            input   [shape=invhouse
+            input   [shape=invhouse fixedsize=shape
                     tooltip="Solution value given into the computation."
                     URL="%(arch_url)s#term-inputs"];
-            output  [shape=house
+            output  [shape=house fixedsize=shape
                     tooltip="Solution value asked from the computation."
                     URL="%(arch_url)s#term-outputs"];
-            inp_out [shape=hexagon label="inp+out"
+            inp_out [shape=hexagon label="inp+out" fixedsize=shape
                     tooltip="Data both given and asked."
                     URL="%(arch_url)s#term-dependency"];
             evicted %(evicted)s
             sfx %(sfx)s
-            sol     [shape=rect style=filled fillcolor=wheat label="with value"
+            sol     [shape=rect fixedsize=shape
+                    style=filled fillcolor=wheat label="with value"
                     tooltip="Data contained in the solution."
                     URL="%(arch_url)s#term-solution"];
-            overwrite [shape=rect style=filled fillcolor=SkyBlue
+            overwrite [shape=rect fixedsize=shape
+                    style=filled fillcolor=SkyBlue
                     tooltip="More than 1 values exist in solution with this name."
                     URL="%(arch_url)s#term-overwrite"];
             DataLabel -> data -> input -> output -> inp_out -> sfx -> evicted -> sol -> overwrite [style=invis];
@@ -1678,6 +1921,27 @@ def legend(
     return plotter.legend(filename, jupyter_render)
 
 
+def save_plot_file_by_sha1(plottable: Plottable, dir_prefix: Path):
+    """Save :term:`plottable` in a fpath generated from sha1 of the `dot`."""
+    from hashlib import sha1
+
+    img_format = "svg"
+
+    dot = plottable.plot()
+
+    ## Derrive image-filename from graph contents.
+    hasher = sha1()
+    hasher.update(str(dot).encode())
+    fname = f"{hasher.hexdigest()}.{img_format}"
+    fpath = dir_prefix / fname
+
+    log.info("Rendering errored %s into file '%s'...", plottable, fpath)
+    dir_prefix.mkdir(parents=True, exist_ok=True)
+    dot.write(fpath, format=img_format)
+
+    return fpath
+
+
 def supported_plot_formats() -> List[str]:
     """return automatically all `pydot` extensions"""
     return [".%s" % f for f in pydot.Dot().formats]
