@@ -22,20 +22,25 @@ from sphinx.domains import Domain, Index, ObjType
 from sphinx.domains.std import StandardDomain
 from sphinx.environment import BuildEnvironment
 from sphinx.ext import doctest as extdoctest
+from sphinx.ext.autodoc import ModuleDocumenter
 from sphinx.locale import _, __
 from sphinx.roles import XRefRole
+from sphinx.util import inspect
 from sphinx.util import logging
 from sphinx.util.console import bold  # pylint: disable=no-name-in-module
 from sphinx.writers.html import HTMLTranslator
 from sphinx.writers.latex import LaTeXTranslator
 
+from graphtik.fnop import FnOp
+from graphtik.plot import default_jupyter_render
+
 from .. import __version__
 
 try:
     import importlib.resources as pkg_resources
 except ImportError:
     # Use backported to PY<3.7 `importlib_resources` lib.
-    import importlib_resources as pkg_resources
+    import importlib_resources as pkg_resources  # noqa
 
 obj_name = "graphtik diagram"
 role_name = "graphtik"
@@ -205,7 +210,7 @@ _doctest_options = extdoctest.DoctestDirective.option_spec
 _img_options = {
     k: v
     for k, v in directives.images.Image.option_spec.items()
-    if k not in ("name", "align", "target")
+    if k not in ("name", "align")
 }
 _fig_options = {
     k: v
@@ -401,13 +406,17 @@ def _should_work(app: Sphinx):
 
 def _run_doctests_on_graphtik_document(app: Sphinx, doctree: nodes.Node):
     """Callback of `doctree-resolved`` event. """
-    docname = app.env.docname
-    from ._graphtikbuilder import get_graphtik_builder
+    try:
+        docname = app.env.docname
+        from ._graphtikbuilder import get_graphtik_builder
 
-    if _should_work(app) and any(doctree.traverse(graphtik_node)):
-        log.info(__("Graphtik-ing document %r..."), docname)
-        graphtik_builder = get_graphtik_builder(app)
-        graphtik_builder.test_doc(docname, doctree)
+        if _should_work(app) and any(doctree.traverse(graphtik_node)):
+            log.info(__("Graphtik-ing document %r..."), docname)
+            graphtik_builder = get_graphtik_builder(app)
+            graphtik_builder.test_doc(docname, doctree)
+    except Exception as ex:
+        log.error("General failure of Graphtik-sphinx extension: %s", ex, exc_info=True)
+        raise
 
 
 class DocFilesPurgatory:
@@ -501,11 +510,38 @@ def _validate_and_apply_configs(app: Sphinx, config: Config):
     )
 
 
-def setup(app: Sphinx):
-    setup.app = app
-    app.require_sphinx("2.0")
-    app.setup_extension("sphinx.ext.doctest")
+def _teach_documenter_about_operations(FuncDocClass):
+    original__can_document_member = FuncDocClass.can_document_member
 
+    def patched__can_document_member(cls, member, membername, isattr, parent):
+        return original__can_document_member(member, membername, isattr, parent) or (
+            isinstance(member, FnOp) and isinstance(parent, ModuleDocumenter)
+        )
+
+    FuncDocClass.can_document_member = classmethod(patched__can_document_member)
+
+
+def process_fnop_signature(app, what, name, obj, options, signature, return_annotation):
+    try:
+        if isinstance(obj, FnOp):
+            fn = obj.fn
+            fn_sig = inspect.signature(fn)
+            signature = str(fn_sig)
+            return_annotation = None
+    except Exception as ex:
+        log.error("Failed `autodoc-process-signature`: %s", ex, exc_info=1)
+    return (signature, return_annotation)
+
+
+def _setup_autodoc(app: Sphinx):
+    app.setup_extension("sphinx.ext.autodoc")
+
+    _teach_documenter_about_operations(app.registry.documenters["function"])
+    app.connect("autodoc-process-signature", process_fnop_signature)
+
+
+def _setup_directive(app: Sphinx):
+    app.setup_extension("sphinx.ext.doctest")
     app.add_config_value(
         "graphtik_graph_formats_by_builder",
         {"html": "svg", "readthedocs": "svg", "latex": "pdf"},
@@ -519,7 +555,7 @@ def setup(app: Sphinx):
     app.add_config_value("graphtik_save_dot_files", None, "html", [bool])
     app.add_config_value(
         "graphtik_zoomable_options",
-        "{controlIconsEnabled: true, fit: true}",
+        default_jupyter_render["svg_pan_zoom_json"],
         "html",
         [str],
     )
@@ -564,6 +600,14 @@ def setup(app: Sphinx):
     # Permanently set this, or else, e.g. +SKIP will not work!
     app.config.trim_doctest_flags = False
 
+
+def setup(app: Sphinx):
+    setup.app = app
+    app.require_sphinx("2.0")
+
+    _setup_autodoc(app)
+    _setup_directive(app)
+
     return {
         "version": __version__,
         "parallel_read_safe": True,
