@@ -3,14 +3,18 @@
 import functools as fnt
 import itertools as itt
 import logging
+import sys
 import textwrap
+from pathlib import Path
 from unittest.mock import MagicMock
 
 import pytest
 
-from graphtik import base, planning, operation, pipeline
-from graphtik.execution import ExecutionPlan, Solution, _OpTask
+from graphtik import base, operation, pipeline, planning
 from graphtik.base import Operation
+from graphtik.jetsam import save_jetsam
+from graphtik.config import debug_enabled
+from graphtik.execution import ExecutionPlan, Solution, _OpTask
 from graphtik.pipeline import Pipeline
 
 
@@ -25,7 +29,7 @@ def test_jetsam_bad_locals(locs, caplog):
         try:
             raise Exception()
         except Exception as ex:
-            base.jetsam(ex, locs, a="a")
+            save_jetsam(ex, locs, a="a")
             raise
 
     assert not hasattr(excinfo.value, "jetsam")
@@ -39,7 +43,7 @@ def test_jetsam_bad_keys(keys, caplog):
         try:
             raise Exception("ABC")
         except Exception as ex:
-            base.jetsam(ex, {}, **keys)
+            save_jetsam(ex, {}, **keys)
 
     assert not hasattr(excinfo.value, "jetsam")
     assert "Suppressed error while annotating exception" not in caplog.text
@@ -52,7 +56,7 @@ def test_jetsam_bad_locals_given(locs, caplog):
         try:
             raise Exception("ABC")
         except Exception as ex:
-            base.jetsam(ex, locs, a="a")
+            save_jetsam(ex, locs, a="a")
             raise
 
     assert not hasattr(excinfo.value, "jetsam")
@@ -68,7 +72,7 @@ def test_jetsam_bad_existing_annotation(annotation, caplog):
             ex.jetsam = annotation
             raise ex
         except Exception as ex:
-            base.jetsam(ex, {}, a="a")
+            save_jetsam(ex, {}, a="a")
             raise
 
     assert excinfo.value.jetsam == {"a": None}
@@ -80,7 +84,7 @@ def test_jetsam_dummy_locals(caplog):
         try:
             raise Exception("ABC")
         except Exception as ex:
-            base.jetsam(ex, {"a": 1}, a="a", bad="bad")
+            save_jetsam(ex, {"a": 1}, a="a", bad="bad")
             raise
 
     assert isinstance(excinfo.value.jetsam, dict)
@@ -99,7 +103,7 @@ def _jetsamed_fn(*args, **kwargs):
         b = 2
         _scream()
     except Exception as ex:
-        base.jetsam(ex, locals(), a="a", b="b")
+        save_jetsam(ex, locals(), a="a", b="b")
         raise
 
 
@@ -117,7 +121,7 @@ def test_jetsam_nested():
             fn = "inner"
             _jetsamed_fn()
         except Exception as ex:
-            base.jetsam(ex, locals(), fn="fn")
+            save_jetsam(ex, locals(), fn="fn")
             raise
 
     def outer():
@@ -127,7 +131,7 @@ def test_jetsam_nested():
             b = 0
             inner()
         except Exception as ex:
-            base.jetsam(ex, locals(), fn="fn")
+            save_jetsam(ex, locals(), fn="fn")
             raise
 
     with pytest.raises(Exception, match="ABC") as excinfo:
@@ -164,7 +168,7 @@ class _ScreamingOperation(Operation):
         ),
         (
             lambda: fnt.partial(
-                ExecutionPlan(*([None] * 6))._handle_task,
+                ExecutionPlan(*([None] * 7))._handle_task,
                 op=_ScreamingOperation(),
                 solution=Solution(MagicMock(), {}),
                 future=_OpTask(_ScreamingOperation(), {}, "solid"),
@@ -185,9 +189,8 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
     assert set(ex.jetsam.keys()) == set(expected_jetsam)
 
 
-@pytest.mark.parametrize(
-    "acallable, expected_jetsam",
-    [
+@pytest.fixture(
+    params=[
         # NO old-stuff Operation(fn=_jetsamed_fn, name="test", needs="['a']", provides=[]),
         (
             lambda: fnt.partial(
@@ -197,7 +200,7 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
         ),
         (
             lambda: fnt.partial(
-                ExecutionPlan(*([None] * 6))._handle_task,
+                ExecutionPlan(*([None] * 7))._handle_task,
                 op=operation(_scream, name="Ah!"),
                 solution=Solution(MagicMock(), {}),
                 future=_OpTask(_ScreamingOperation(), {}, "solid"),
@@ -206,7 +209,7 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
         ),
         (
             lambda: fnt.partial(
-                ExecutionPlan(*([None] * 6)).execute, named_inputs=None
+                ExecutionPlan(*([None] * 7)).execute, named_inputs=None
             ),
             ["solution"],
         ),
@@ -216,19 +219,40 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
                 named_inputs=None,
                 outputs="bad",
             ),
-            "network plan solution outputs".split(),
+            "network plan solution pipeline outputs plot_fpath".split(),
         ),
-    ],
+    ]
 )
-def test_jetsam_sites_scream(acallable, expected_jetsam):
+def failing_jetsam(request):
+    return request.param
+
+
+def test_jetsam_sites_scream(failing_jetsam):
     # Check jetsams when the site fails.
+    acallable, expected_jetsam = failing_jetsam
     acallable = acallable()
     with pytest.raises(Exception) as excinfo:
         acallable()
 
     ex = excinfo.value
     assert hasattr(ex, "jetsam"), acallable
-    assert set(ex.jetsam.keys()) == set(expected_jetsam)
+    assert set(ex.jetsam.keys()) == set(i for i in expected_jetsam if i != "plot_fpath")
+
+
+@pytest.mark.xfail(
+    sys.version_info < (3, 8), reason="unlink(missing_ok) param introduced in PY3.8"
+)
+def test_jetsam_saves_plot(failing_jetsam):
+    acallable, expected_jetsam = failing_jetsam
+    if "plot_fpath" in expected_jetsam:
+        acallable = acallable()
+        with debug_enabled(True), pytest.raises(Exception) as excinfo:
+            acallable()
+
+        ex = excinfo.value
+        fp = Path(ex.jetsam["plot_fpath"])
+        assert fp.exists()
+        fp.unlink(missing_ok=True)
 
 
 ###############
