@@ -3,17 +3,33 @@
 """General :term:`network` & :term:`execution` tests. """
 import math
 import re
+import sys
 from operator import add, floordiv, mul, sub
 
+import networkx as nx
 import pytest
 
-from graphtik import NO_RESULT, compose, planning, operation, optional, vararg
+from graphtik import (
+    NO_RESULT,
+    compose,
+    modify,
+    planning,
+    operation,
+    optional,
+    vararg,
+    varargs,
+    sfxed,
+    sfx,
+)
 from graphtik.base import IncompleteExecutionError
-from graphtik.config import evictions_skipped, operations_endured
+from graphtik.config import debug_enabled, evictions_skipped, operations_endured
 
 from .helpers import addall
 
 
+pytestmark = pytest.mark.usefixtures("log_levels")
+
+
 def scream(*args, **kwargs):
     raise AssertionError(f"Must not have run!\n    args: {args}\n  kwargs: {kwargs}")
 
@@ -106,10 +122,28 @@ def test_smoke_test():
         pipeline.compute({"sum_ab": 1, "b": 2}, ["b", "bad_node"])
 
 
+def test_cycle_tip():
+    pipe = compose(..., operation(str, "cyclic1", "a", "a"))
+    with pytest.raises(nx.NetworkXUnfeasible, match="TIP:"):
+        pipe.compute()
+    pipe = compose(
+        ..., operation(str, "cyclic1", "a", "b"), operation(str, "cyclic2", "b", "a")
+    )
+    with evictions_skipped(True), pytest.raises(nx.NetworkXUnfeasible, match="TIP:"):
+        pipe.compute()
+
+
 def test_aliases(exemethod):
-    aliased = operation(lambda: "A", name="op1", provides="a", aliases={"a": "b"})
-    assert aliased.provides == ("a",)
-    assert aliased.op_provides == ("a", "b")
+    aliased = operation(
+        lambda: ("A", "B"),
+        name="op1",
+        provides=["a", sfxed("s1", "foo"), sfxed("s2", "foo", implicit=1)],
+        aliases={"a": "b", "s1": "S1", "s2": "S2"},
+    )
+    assert aliased.provides == ("a", sfxed("s1", "foo"), sfxed("s2", "foo"))
+    assert set(aliased.op_provides) == set(
+        ("a", "b", "S1", "S2", sfxed("s1", "foo"), sfxed("s2", "foo"))
+    )
 
     op = compose(
         "test_net",
@@ -117,7 +151,7 @@ def test_aliases(exemethod):
         operation(lambda x: x * 2, name="op2", needs="b", provides="c"),
         parallel=exemethod,
     )
-    assert op() == {"a": "A", "b": "A", "c": "AA"}
+    assert op() == {"a": "A", "s1": "B", "b": "A", "S1": "B", "c": "AA"}
 
 
 def test_node_predicate_based_prune():
@@ -891,3 +925,47 @@ def test_rescheduling_NO_RESULT(exemethod):
     assert "x1 partial-ops" in str(sol.check_if_incomplete())
     with pytest.raises(IncompleteExecutionError, match="x1 partial-ops"):
         assert sol.scream_if_incomplete()
+
+
+def test_jetsam_n_plot_with_DEBUG():
+    pipe = compose(
+        "mix",
+        operation(
+            str,
+            "FUNC",
+            needs=[
+                "a",
+                sfxed("b", "foo", keyword="bb"),
+                modify("c", implicit=1),
+                sfxed("d", "bar", implicit=1),
+                vararg("e"),
+                varargs("f"),
+            ],
+            provides=[
+                "A",
+                sfxed("b", "FOO", keyword="bb"),
+                modify("C", implicit=1),
+                sfxed("d", "BAR", implicit=1),
+                sfx("FOOBAR"),
+            ],
+            aliases={"A": "aaa", "b": "bbb", "C": "ccc", "d": "ddd"},
+        ),
+    )
+
+    with debug_enabled(True), pytest.raises(ValueError, match="^Unsolvable"):
+        pipe.compute()
+    with debug_enabled(True), pytest.raises(
+        ValueError, match="^Failed matching inputs <=> needs"
+    ) as exc:
+        pipe.compute(
+            {
+                "a": 1,
+                sfxed("b", "foo"): 2,
+                "c": 3,
+                sfxed("d", "bar"): 4,
+                "e": 5,
+                "f": [6, 7],
+            }
+        )
+
+    exc.value.jetsam.plot_fpath.unlink()
