@@ -12,7 +12,10 @@ from graphtik import NO_RESULT, compose, operation, sfxed, vararg
 from graphtik.base import RenArgs
 from graphtik.config import solution_layered
 from graphtik.execution import task_context
-from graphtik.modifier import accessor, dep_renamed, jsonp
+from graphtik.modifier import accessor, dep_renamed, modify
+
+
+pytestmark = pytest.mark.usefixtures("log_levels")
 
 
 def test_solution_accessor_simple():
@@ -34,8 +37,8 @@ def test_jsonp_disabled():
     no_jsonp_res = operation(
         fn=None,
         name="copy a+b-->A+BB",
-        needs=[jsonp("inputs/a"), jsonp("inputs/b")],
-        provides=["RESULTS/A", jsonp("RESULTS/BB", False)],
+        needs=[modify("inputs/a"), modify("inputs/b")],
+        provides=["RESULTS/A", modify("RESULTS/BB", False)],
     )
     assert "provides=['RESULTS/A'($), RESULTS/BB" in str(no_jsonp_res)
     res = compose("", no_jsonp_res).compute({"inputs": {"a": 1, "b": 2}})
@@ -43,7 +46,7 @@ def test_jsonp_disabled():
 
     no_jsonp_inp = operation(
         name="copy a+b-->A+BB",
-        needs=["inputs/a", jsonp("inputs/b", False)],
+        needs=["inputs/a", modify("inputs/b", False)],
         provides=["RESULTS/A", "RESULTS/BB"],
     )()
     assert "needs=['inputs/a'($), inputs/b]" in str(no_jsonp_inp)
@@ -103,7 +106,7 @@ def test_jsonp_and_conveyor_fn_complex_LAYERED(solution_layered_true):
         operation(
             name="op1",
             needs=["i/a", "i/a"],  # dupe jsonp needs
-            provides=["r/a", jsonp("a")],
+            provides=["r/a", modify("a")],
         )(),
         operation(
             lambda x: (x, 2 * x), name="op2", needs=["r/a"], provides=["r/A", "r/AA"],
@@ -128,7 +131,7 @@ def test_jsonp_and_conveyor_fn_complex_NOT_LAYERED(solution_layered_false):
         operation(
             name="op1",
             needs=["i/a", "i/a"],  # dupe jsonp needs
-            provides=["r/a", jsonp("a")],
+            provides=["r/a", modify("a")],
         )(),
         operation(
             lambda x: (x, 2 * x), name="op2", needs=["r/a"], provides=["r/A", "r/AA"],
@@ -177,7 +180,7 @@ def test_network_nest_subdocs_LAYERED(solution_layered_true):
             return True
         # if is_sfxed(dep):
         #     return modifier_withset(
-        #         dep, sfx_list=[f"{ra.parent.name}.{s}" for s in dep.sfx_list]
+        #         dep, sfx_list=[f"{ra.parent.name}.{s}" for s in dep._sfx_list]
         #     )
         if dep == "daily_tasks":
             return dep_renamed(dep, lambda n: f"{n}/{ra.parent.name}")
@@ -340,7 +343,7 @@ def test_network_nest_subdocs_NOT_LAYERED(solution_layered_false):
             return True
         # if is_sfxed(dep):
         #     return modifier_withset(
-        #         dep, sfx_list=[f"{ra.parent.name}.{s}" for s in dep.sfx_list]
+        #         dep, sfx_list=[f"{ra.parent.name}.{s}" for s in dep._sfx_list]
         #     )
         if dep == "daily_tasks":
             return dep_renamed(dep, lambda n: f"{n}/{ra.parent.name}")
