@@ -104,13 +104,19 @@ def test_op_label_template_full():
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="&lt;op &quot; &#9; tooltip&gt;" HREF="http://op_url.com_label_" TARGET="_self"
                 ><FONT COLOR="blue"><B>OP:</B> <I>the op</I></FONT></TD>
-                <TD BORDER="1" SIDES="b"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
-                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-endured" TARGET="_blank"
-                            ><FONT FACE="monospace" COLOR="white"><B>!</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-partial-outputs" TARGET="_blank"
-                            ><FONT FACE="monospace" COLOR="white"><B>?</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-parallel-execution" TARGET="_blank"
-                            ><FONT FACE="monospace" COLOR="white"><B>|</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-marshalling" TARGET="_blank"
-                            ><FONT FACE="monospace" COLOR="white"><B>&amp;</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-returns-dictionary" TARGET="_blank"
-                            ><FONT FACE="monospace" COLOR="white"><B>}</B></FONT></TD></TR>
+                <TD BORDER="1" SIDES="b" ALIGN="right"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2" ALIGN="right">
+                        <TR>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-endured" TARGET="_top"
+                            ><FONT FACE="monospace" COLOR="white"><B>!</B></FONT></TD>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-partial-outputs" TARGET="_top"
+                            ><FONT FACE="monospace" COLOR="white"><B>?</B></FONT></TD>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-parallel-execution" TARGET="_top"
+                            ><FONT FACE="monospace" COLOR="white"><B>|</B></FONT></TD>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-marshalling" TARGET="_top"
+                            ><FONT FACE="monospace" COLOR="white"><B>&amp;</B></FONT></TD>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="12" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-returns-dictionary" TARGET="_top"
+                            ><FONT FACE="monospace" COLOR="white"><B>}</B></FONT></TD>
+                        </TR>
                     </TABLE></TD>
             </TR><TR>
                 <TD COLSPAN="2" ALIGN="left" TOOLTIP="&lt;fn&#10;tooltip&gt;" HREF="http://fn_url.com/quoto_and" TARGET="_top"
@@ -136,9 +142,9 @@ def test_op_label_template_empty():
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
+                <TD BORDER="1" SIDES="b" ALIGN="left"
                 ></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"></TD>
             </TR>
         </TABLE>>
         """
@@ -157,11 +163,11 @@ def test_op_label_template_fn_empty():
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
+                <TD BORDER="1" SIDES="b" ALIGN="left"
                 ><B>OP:</B> <I>op</I></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"></TD>
             </TR><TR>
-                <TD COLSPAN="2" ALIGN="left" TARGET=""
+                <TD COLSPAN="2" ALIGN="left"
                 ><B>FN:</B> fn</TD>
             </TR>
         </TABLE>>
@@ -188,9 +194,9 @@ def test_op_label_template_nones():
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
+                <TD BORDER="1" SIDES="b" ALIGN="left"
                 ></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"></TD>
             </TR>
         </TABLE>>
         """
@@ -547,24 +553,30 @@ def test_node_dot_str0(dot_str_pipeline):
         digraph graph_ {
         fontname=italic;
         label=<graph>;
-        <edge> [label=edge, margin="0.04,0.02", shape=invhouse];
-        <digraph&#58; strict> [label="digraph: strict", margin="0.04,0.02", shape=invhouse];
+        <edge> [fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD>edge</TD></TR>
+                </TABLE>>, shape=invhouse, tooltip="(input)"];
+        <digraph&#58; strict> [fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD>digraph: strict</TD></TR>
+                </TABLE>>, shape=invhouse, tooltip="(input)"];
         <node> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>node</I></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"></TD>
             </TR><TR>
                 <TD COLSPAN="2" ALIGN="left" TOOLTIP="Same as a + b." TARGET="_top"
                 ><B>FN:</B> &lt;built-in function add&gt;</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<node>];
-        <&lt;graph&gt;> [label=<graph>, margin="0.04,0.02", shape=house];
+        <&lt;graph&gt;> [fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD><graph></TD></TR>
+                </TABLE>>, shape=house, tooltip="(output)"];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"></TD>
             </TR><TR>
                 <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET="_top"
                 ><B>FN:</B> test.test_plot.func</TD>
@@ -594,20 +606,30 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
     overlay.add_node(hidden_op, no_plot=True)
     overlay.graph["graphviz.splines"] = "ortho"
 
-    exp = """
+    exp = r"""
         digraph solution_x5_nodes {
         fontname=italic;
         splines=ortho;
         subgraph "cluster_after pruning" {
         label=<after pruning>;
-        <edge> [fillcolor=wheat, label=edge, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
-        <digraph&#58; strict> [fillcolor=wheat, label="digraph: strict", margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
-        <&lt;graph&gt;> [fillcolor=SkyBlue, label=<graph>, margin="0.04,0.02", shape=house, style=filled, tooltip="(None)"];
+        <edge> [fillcolor=wheat, fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD>edge</TD></TR>
+                </TABLE>>, shape=invhouse, style=filled, tooltip="(input)\n(int) 1"];
+        <digraph&#58; strict> [fillcolor=wheat, fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD>digraph: strict</TD></TR>
+                </TABLE>>, shape=invhouse, style=filled, tooltip="(input)\n(int) 2"];
+        <&lt;graph&gt;> [fillcolor=SkyBlue, fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                    <TR><TD><graph></TD></TR>
+                </TABLE>>, shape=house, style=filled, tooltip="(output)\n(None)\n(overridden)"];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
-                <TD BORDER="1" SIDES="b"></TD>
+                <TD BORDER="1" SIDES="b" ALIGN="right"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2" ALIGN="right">
+                        <TR>
+                            <TD STYLE="rounded" HEIGHT="22" WIDTH="14" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#00bbbb" TITLE="computation order" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-steps" TARGET="_top"><FONT FACE="monospace" COLOR="white"><B>0</B></FONT></TD>
+                        </TR>
+                    </TABLE></TD>
             </TR><TR>
                 <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET="_top"
                 ><B>FN:</B> test.test_plot.func</TD>
@@ -649,6 +671,75 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
     assert "<node>" not in dot_str
 
 
+def test_step_badge():
+    p = compose(
+        "",
+        operation(str, "0", provides="b"),
+        operation(str, "1", needs="b", provides="c"),
+    )
+    sol = p.compute(outputs="c")
+    dot_str = str(sol.plot())
+    print(dot_str)
+    exp = r"""
+    digraph solution_x4_nodes {
+    fontname=italic;
+    <0> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
+        <TR>
+            <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;0&#x27;, provides=[&#x27;b&#x27;], fn=&#x27;str&#x27;)" TARGET="_top"
+            ><B>OP:</B> <I>0</I></TD>
+            <TD BORDER="1" SIDES="b" ALIGN="right"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2" ALIGN="right">
+                    <TR>
+                        <TD STYLE="rounded" HEIGHT="22" WIDTH="14" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#00bbbb" TITLE="computation order" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-steps" TARGET="_top"><FONT FACE="monospace" COLOR="white"><B>0</B></FONT></TD>
+                    </TR>
+                </TABLE></TD>
+        </TR><TR>
+            <TD COLSPAN="2" ALIGN="left" TARGET="_top"
+            ><B>FN:</B> builtins.str</TD>
+        </TR>
+    </TABLE>>, shape=plain, tooltip=<0>];
+    <b> [color="#006666", fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                <TR><TD>b</TD>
+                <TD STYLE="rounded" CELLSPACING="2" CELLPADDING="4" BGCOLOR="#00bbbb" TITLE="computation order" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-steps" TARGET="_top"
+                ><FONT FACE="monospace" COLOR="white"><B>2</B></FONT></TD></TR>
+            </TABLE>>, penwidth=3, shape=rect, style=dashed, tooltip="(to evict)\n(evicted)"];
+    <1> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
+        <TR>
+            <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;1&#x27;, needs=[&#x27;b&#x27;], provides=[&#x27;c&#x27;], fn=&#x27;str&#x27;)" TARGET="_top"
+            ><B>OP:</B> <I>1</I></TD>
+            <TD BORDER="1" SIDES="b" ALIGN="right"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2" ALIGN="right">
+                    <TR>
+                        <TD STYLE="rounded" HEIGHT="22" WIDTH="14" FIXEDSIZE="true" VALIGN="BOTTOM" BGCOLOR="#00bbbb" TITLE="computation order" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-steps" TARGET="_top"><FONT FACE="monospace" COLOR="white"><B>1</B></FONT></TD>
+                    </TR>
+                </TABLE></TD>
+        </TR><TR>
+            <TD COLSPAN="2" ALIGN="left" TARGET="_top"
+            ><B>FN:</B> builtins.str</TD>
+        </TR>
+    </TABLE>>, shape=plain, tooltip=<1>];
+    <c> [fillcolor=wheat, fixedsize=shape, label=<<TABLE CELLBORDER="0" CELLSPACING="0" BORDER="0">
+                <TR><TD>c</TD></TR>
+            </TABLE>>, shape=house, style=filled, tooltip="(output)\n(str)"];
+    <0> -> <b>  [headport=n, tailport=s];
+    <b> -> <1>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+    <1> -> <c>  [headport=n, tailport=s];
+    legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_blank];
+    }
+    """
+    assert _striplines(dot_str) == _striplines(exp)
+
+
+def test_node_edge_defaults():
+    dot = compose("defs", operation(str, "a", "b")).plot(
+        theme={"node_defaults": {"color": "red"}}
+    )
+    assert "color=red" in str(dot)
+
+    dot = compose("defs", operation(str, "a", "b")).plot(
+        theme={"edge_defaults": {"color": "red"}}
+    )
+    assert "color=red" in str(dot)
+
+
 def test_combine_clusters():
     p1 = compose(
         "op1",
@@ -667,3 +758,7 @@ def test_combine_clusters():
     dot: pydot.Dot = merged_graph.plot()
     assert dot.get_subgraph(f"cluster_{p1.name}")
     assert dot.get_subgraph(f"cluster_{p2.name}")
+
+
+def test_degenerate_pipeline():
+    compose("defs", operation(str, "a")).plot()
